@@ -40,7 +40,7 @@ import java.util.Queue;
 import java.util.Set;
 import java.util.TreeSet;
 import java.util.concurrent.ConcurrentLinkedQueue;
-import java.util.concurrent.atomic.AtomicInteger;
+import java.util.concurrent.atomic.AtomicReference;
 
 /**
  * A hierarchical {@link ServiceRegistry} implementation.
@@ -53,11 +53,8 @@ import java.util.concurrent.atomic.AtomicInteger;
  *
  * <li>Adding a factory method. A factory method should have a name that starts with 'create', and have a non-void return type. For example, <code>protected SomeService createSomeService() { ....
  * }</code>. Parameters are injected using services from this registry or its parents. Parameter of type {@link ServiceRegistry} will receive the service registry that owns the service. Parameter of
- * type {@code List<T>} will receive all services of type T, if any. Note that factory methods with a single parameter and an return type equal to that parameter type are interpreted as decorator
- * methods.</li>
- *
- * <li>Adding a decorator method. A decorator method should have a name that starts with 'decorate', take a single parameter, and a have return type equal to the parameter type. Before invoking the
- * method, the parameter is located in the parent service registry and then passed to the method.</li>
+ * type {@code List<T>} will receive all services of type T, if any. If a parameter has the same type as the return type of the factory method, then that parameter will be looked up in the parent registry.
+ * This allows decorating services.</li>
  *
  * <li>Adding a configure method. A configure method should be called 'configure', take a {@link ServiceRegistration} parameter, and a have a void return type. Additional parameters are injected using
  * services from this registry or its parents.</li>
@@ -72,7 +69,7 @@ import java.util.concurrent.atomic.AtomicInteger;
  * <p>Service registries are arranged in a hierarchy. If a service of a given type cannot be located, the registry uses its parent registry, if any, to locate the service.</p>
  */
 public class DefaultServiceRegistry implements ServiceRegistry, Closeable {
-    private enum State {INIT, IN_USE, CLOSING, CLOSED};
+    private enum State {INIT, STARTED, CLOSED};
     private final static ServiceRegistry[] NO_PARENTS = new ServiceRegistry[0];
     private final static Service[] NO_DEPENDENTS = new Service[0];
     private final static Object[] NO_PARAMS = new Object[0];
@@ -83,10 +80,7 @@ public class DefaultServiceRegistry implements ServiceRegistry, Closeable {
     private final String displayName;
     private final ServiceProvider thisAsServiceProvider;
 
-    private State state = State.INIT;
-    private final Object stateLock = new Object();
-    private final AtomicInteger inProgress = new AtomicInteger(0);
-
+    private AtomicReference<State> state = new AtomicReference<State>(State.INIT);
 
     public DefaultServiceRegistry() {
         this(null, NO_PARENTS);
@@ -167,7 +161,7 @@ public class DefaultServiceRegistry implements ServiceRegistry, Closeable {
             if (parentServices == null) {
                 throw new ServiceLookupException(String.format("Cannot use decorator method %s.%s() when no parent registry is provided.", type.getSimpleName(), method.getName()));
             }
-            ownServices.add(new DecoratorMethodService(this, target, method));
+            ownServices.add(new FactoryMethodService(this, target, method));
         }
         for (ServiceMethod method : methods.factories) {
             ownServices.add(new FactoryMethodService(this, target, method));
@@ -212,7 +206,7 @@ public class DefaultServiceRegistry implements ServiceRegistry, Closeable {
     }
 
     private void assertMutable() {
-        if (state != State.INIT) {
+        if (state.get() != State.INIT) {
             throw new IllegalStateException("Cannot add provide to service registry " + this + " as it is no longer mutable");
         }
     }
@@ -255,86 +249,55 @@ public class DefaultServiceRegistry implements ServiceRegistry, Closeable {
      * Closes all services for this registry. For each service, if the service has a public void close() or stop() method, that method is called to close the service.
      */
     public void close() {
-        if (state == State.CLOSED || state == State.CLOSING) {
-            return;
-        }
-
-        synchronized (stateLock) {
-            if (state == State.CLOSED || state == State.CLOSING) {
-                return;
-            }
-            state = State.CLOSING;
-            waitForPendingRequests();
-            CompositeStoppable.stoppable(allServices).stop();
-            state = State.CLOSED;
-        }
-    }
-
-    private void waitForPendingRequests() {
-        while (inProgress.get() != 0) {
-            try {
-                stateLock.wait();
-            } catch (InterruptedException e) {
-                throw new RuntimeException(e);
-            }
-        }
-    }
-
-    private void newRequestInProgress() {
         noLongerMutable();
-        if (state == State.CLOSED) {
+        if (state.compareAndSet(State.STARTED, State.CLOSED)) {
+            CompositeStoppable.stoppable(allServices).stop();
+        }
+    }
+
+    private void serviceRequested() {
+        noLongerMutable();
+        if (state.get() == State.CLOSED) {
             throw new IllegalStateException(String.format("%s has been closed.", getDisplayName()));
         }
-        inProgress.incrementAndGet();
     }
 
     private void noLongerMutable() {
-        if (state == State.INIT) {
-            synchronized (stateLock) {
-                if (state == State.INIT) {
-                    state = State.IN_USE;
-                    ownServices.noLongerMutable();
-                }
-            }
-        }
-    }
-
-    private void requestFinished() {
-        boolean noMoreRequests = inProgress.decrementAndGet() == 0;
-        if (noMoreRequests && state == State.CLOSING) {
-            synchronized (stateLock) {
-                stateLock.notify();
-            }
+        if (state.compareAndSet(State.INIT, State.STARTED)) {
+            ownServices.noLongerMutable();
         }
     }
 
     public boolean isClosed() {
-        return state == State.CLOSED;
+        return state.get() == State.CLOSED;
     }
 
     public <T> T get(Class<T> serviceType) throws UnknownServiceException, ServiceLookupException {
         return serviceType.cast(get((Type) serviceType));
     }
 
+    @Override
     public Object get(Type serviceType) throws UnknownServiceException, ServiceLookupException {
-        assertValidServiceType(unwrap(serviceType));
-        Service provider = getService(serviceType);
-        Object instance = provider == null ? null : provider.get();
+        Object instance = find(serviceType);
         if (instance == null) {
             throw new UnknownServiceException(serviceType, String.format("No service of type %s available in %s.", format(serviceType), getDisplayName()));
         }
         return instance;
     }
 
-    private Service getService(Type serviceType) {
-        try {
-            newRequestInProgress();
-            return find(serviceType, allServices);
-        } finally {
-            requestFinished();
-        }
+    @Override
+    public Object find(Type serviceType) throws ServiceLookupException {
+        assertValidServiceType(unwrap(serviceType));
+        Service provider = getService(serviceType);
+        return provider == null ? null : provider.get();
     }
 
+    private Service getService(Type serviceType) {
+        serviceRequested();
+        return find(serviceType, allServices);
+    }
+
+    @Override
     public <T> Factory<T> getFactory(Class<T> type) {
         assertValidServiceType(type);
         Service provider = getFactoryService(type);
@@ -346,14 +309,11 @@ public class DefaultServiceRegistry implements ServiceRegistry, Closeable {
     }
 
     private Service getFactoryService(Class<?> serviceType) {
-        try {
-            newRequestInProgress();
-            return allServices.getFactory(serviceType);
-        } finally {
-            requestFinished();
-        }
+        serviceRequested();
+        return allServices.getFactory(serviceType);
     }
 
+    @Override
     public <T> List<T> getAll(Class<T> serviceType) throws ServiceLookupException {
         assertValidServiceType(serviceType);
         List<T> services = new ArrayList<T>();
@@ -362,12 +322,8 @@ public class DefaultServiceRegistry implements ServiceRegistry, Closeable {
     }
 
     private <T> void collectServices(Class<T> serviceType, List<Service> results) {
-        try {
-            newRequestInProgress();
-            allServices.getAll(serviceType, results);
-        } finally {
-            requestFinished();
-        }
+        serviceRequested();
+        allServices.getAll(serviceType, results);
     }
 
     private static class InstanceUnpackingList<T> extends AbstractList<Service> {
@@ -776,6 +732,16 @@ public class DefaultServiceRegistry implements ServiceRegistry, Closeable {
                 Type paramType = parameterTypes[i];
                 if (paramType.equals(ServiceRegistry.class)) {
                     paramServices[i] = owner.getThisAsService();
+                } else if (paramType.equals(serviceType)) {
+                    Service paramProvider = owner.find(paramType, owner.parentServices);
+                    if (paramProvider == null) {
+                        throw new ServiceCreationException(String.format("Cannot create service of type %s using %s.%s() as required service of type %s is not available in parent registries.",
+                            format(serviceType),
+                            getFactory().getDeclaringClass().getSimpleName(),
+                            getFactory().getName(),
+                            format(paramType)));
+                    }
+                    paramServices[i] = paramProvider;
                 } else {
                     Service paramProvider;
                     try {
@@ -944,63 +910,6 @@ public class DefaultServiceRegistry implements ServiceRegistry, Closeable {
         }
     }
 
-    private static class DecoratorMethodService extends SingletonService {
-        private final ServiceMethod method;
-        private Object target;
-        private Service paramServices;
-
-        public DecoratorMethodService(DefaultServiceRegistry owner, Object target, ServiceMethod method) {
-            super(owner, method.getServiceType());
-            this.target = target;
-            this.method = method;
-        }
-
-        public String getDisplayName() {
-            return "Service " + format(method.getServiceType()) + " at " + method.getOwner().getSimpleName() + "." + method.getName() + "()";
-        }
-
-        @Override
-        protected void bind() {
-            Type paramType = method.getParameterTypes()[0];
-            paramServices = owner.find(paramType, owner.parentServices);
-            if (paramServices == null) {
-                throw new ServiceCreationException(String.format("Cannot create service of type %s using %s.%s() as required service of type %s is not available in parent registries.",
-                    format(method.getServiceType()),
-                    method.getOwner().getSimpleName(),
-                    method.getName(),
-                    format(paramType)));
-            }
-        }
-
-        @Override
-        protected Object create() {
-            Object param = paramServices.get();
-            Object result;
-            try {
-                result = method.invoke(target, param);
-            } catch (Exception e) {
-                throw new ServiceCreationException(String.format("Could not create service of type %s using %s.%s().",
-                    format(method.getServiceType()),
-                    method.getOwner().getSimpleName(),
-                    method.getName()),
-                    e);
-            }
-            try {
-                if (result == null) {
-                    throw new ServiceCreationException(String.format("Could not create service of type %s using %s.%s() as this method returned null.",
-                        format(method.getServiceType()),
-                        method.getOwner().getSimpleName(),
-                        method.getName()));
-                }
-                return result;
-            } finally {
-                // Can discard state required to create instance
-                paramServices = null;
-                target = null;
-            }
-        }
-    }
-
     private static class CompositeServiceProvider implements ServiceProvider {
         private final ServiceProvider[] serviceProviders;
 
