@@ -15,15 +15,12 @@
  */
 package org.gradle.util;
 
-import com.google.common.base.Equivalence;
-import com.google.common.base.Function;
 import com.google.common.base.Joiner;
+import com.google.common.base.Preconditions;
 import com.google.common.collect.ImmutableList;
 import com.google.common.collect.ImmutableListMultimap;
-import com.google.common.collect.ImmutableSet;
-import com.google.common.collect.Iterables;
+import com.google.common.collect.Lists;
 import org.gradle.api.Action;
-import org.gradle.api.Nullable;
 import org.gradle.api.Transformer;
 import org.gradle.api.specs.Spec;
 import org.gradle.internal.Cast;
@@ -31,8 +28,25 @@ import org.gradle.internal.Factory;
 import org.gradle.internal.Pair;
 import org.gradle.internal.Transformers;
 
+import javax.annotation.Nullable;
 import java.lang.reflect.Array;
-import java.util.*;
+import java.util.AbstractList;
+import java.util.ArrayList;
+import java.util.Arrays;
+import java.util.Collection;
+import java.util.Collections;
+import java.util.Comparator;
+import java.util.Enumeration;
+import java.util.HashMap;
+import java.util.HashSet;
+import java.util.Iterator;
+import java.util.LinkedHashMap;
+import java.util.LinkedHashSet;
+import java.util.LinkedList;
+import java.util.List;
+import java.util.Map;
+import java.util.NoSuchElementException;
+import java.util.Set;
 
 import static org.gradle.internal.Cast.cast;
 
@@ -68,6 +82,7 @@ public abstract class CollectionUtils {
         return Cast.uncheckedCast(input);
     }
 
+    @Nullable
     public static <T> T findFirst(Iterable<? extends T> source, Spec<? super T> filter) {
         for (T item : source) {
             if (filter.isSatisfiedBy(item)) {
@@ -78,6 +93,7 @@ public abstract class CollectionUtils {
         return null;
     }
 
+    @Nullable
     public static <T> T findFirst(T[] source, Spec<? super T> filter) {
         for (T thing : source) {
             if (filter.isSatisfiedBy(thing)) {
@@ -105,11 +121,11 @@ public abstract class CollectionUtils {
     }
 
     public static <T> List<T> filter(List<? extends T> list, Spec<? super T> filter) {
-        return filter(list, new LinkedList<T>(), filter);
+        return filter(list, Lists.<T>newArrayListWithCapacity(list.size()), filter);
     }
 
     public static <T> List<T> filter(T[] array, Spec<? super T> filter) {
-        return filter(Arrays.asList(array), new LinkedList<T>(), filter);
+        return filter(Arrays.asList(array), Lists.<T>newArrayListWithCapacity(array.length), filter);
     }
 
 
@@ -552,6 +568,30 @@ public abstract class CollectionUtils {
         return string.toString();
     }
 
+    /**
+     * Partition given Collection into a Pair of Collections.
+     *
+     * <pre>Left</pre> Collection containing entries that satisfy the given predicate
+     * <pre>Right</pre> Collection containing entries that do NOT satisfy the given predicate
+     */
+    public static <T> Pair<Collection<T>, Collection<T>> partition(Iterable<T> items, Spec<? super T> predicate) {
+        Preconditions.checkNotNull(items, "Cannot partition null Collection");
+        Preconditions.checkNotNull(predicate, "Cannot apply null Spec when partitioning");
+
+        Collection<T> left = new LinkedList<T>();
+        Collection<T> right = new LinkedList<T>();
+
+        for (T item : items) {
+            if (predicate.isSatisfiedBy(item)) {
+                left.add(item);
+            } else {
+                right.add(item);
+            }
+        }
+
+        return Pair.of(left, right);
+    }
+
     public static class InjectionStep<T, I> {
         private final T target;
         private final I item;
@@ -587,7 +627,7 @@ public abstract class CollectionUtils {
         return target;
     }
 
-    public static <K, V> ImmutableListMultimap<K, V> groupBy(Iterable<? extends V> iterable, Transformer<? extends K, V> grouper) {
+    public static <K, V> Map<K, Collection<V>> groupBy(Iterable<? extends V> iterable, Transformer<? extends K, V> grouper) {
         ImmutableListMultimap.Builder<K, V> builder = ImmutableListMultimap.builder();
 
         for (V element : iterable) {
@@ -595,7 +635,7 @@ public abstract class CollectionUtils {
             builder.put(key, element);
         }
 
-        return builder.build();
+        return builder.build().asMap();
     }
 
     public static <T> Iterable<? extends T> unpack(final Iterable<? extends Factory<? extends T>> factories) {
@@ -626,20 +666,6 @@ public abstract class CollectionUtils {
         return list.isEmpty() ? null : list;
     }
 
-    public static <T> List<T> dedup(Iterable<T> source, final Equivalence<? super T> equivalence) {
-        Iterable<Equivalence.Wrapper<T>> wrappers = Iterables.transform(source, new Function<T, Equivalence.Wrapper<T>>() {
-            public Equivalence.Wrapper<T> apply(@Nullable T input) {
-                return equivalence.wrap(input);
-            }
-        });
-        Set<Equivalence.Wrapper<T>> deduped = ImmutableSet.copyOf(wrappers);
-        return ImmutableList.copyOf(Iterables.transform(deduped, new Function<Equivalence.Wrapper<T>, T>() {
-            public T apply(Equivalence.Wrapper<T> input) {
-                return input.get();
-            }
-        }));
-    }
-
     public static String asCommandLine(Iterable<String> arguments) {
         return Joiner.on(" ").join(collect(arguments, Transformers.asSafeCommandLineArgument()));
     }
