@@ -16,6 +16,7 @@
 package org.gradle.api.plugins.quality;
 
 import com.google.common.util.concurrent.Callables;
+import org.apache.tools.ant.Main;
 import org.gradle.api.Action;
 import org.gradle.api.artifacts.Configuration;
 import org.gradle.api.artifacts.DependencySet;
@@ -52,6 +53,11 @@ public class JDependPlugin extends AbstractCodeQualityPlugin<JDepend> {
         return JDepend.class;
     }
 
+    @Override
+    protected void configureConfiguration(Configuration configuration) {
+        configureDefaultDependencies(configuration);
+    }
+
     @Override
     protected CodeQualityExtension createExtension() {
         extension = project.getExtensions().create("jdepend", JDependExtension.class);
@@ -61,8 +67,7 @@ public class JDependPlugin extends AbstractCodeQualityPlugin<JDepend> {
 
     @Override
     protected void configureTaskDefaults(JDepend task, String baseName) {
-        Configuration configuration = project.getConfigurations().getAt("jdepend");
-        configureDefaultDependencies(configuration);
+        Configuration configuration = project.getConfigurations().getAt(getConfigurationName());
         configureTaskConventionMapping(configuration, task);
         configureReportsConventionMapping(task, baseName);
     }
@@ -72,7 +77,9 @@ public class JDependPlugin extends AbstractCodeQualityPlugin<JDepend> {
             @Override
             public void execute(DependencySet dependencies) {
                 dependencies.add(project.getDependencies().create("jdepend:jdepend:" + extension.getToolVersion()));
-                dependencies.add(project.getDependencies().create("org.apache.ant:ant-jdepend:1.9.6"));
+                Class<Main> antMain = Main.class;
+                String antVersion = antMain.getPackage().getImplementationVersion();
+                dependencies.add(project.getDependencies().create("org.apache.ant:ant-jdepend:" + antVersion));
             }
         });
     }
