@@ -19,6 +19,8 @@ import groovy.lang.Closure;
 import groovy.lang.DelegatesTo;
 import org.gradle.api.Action;
 import org.gradle.api.Incubating;
+import org.gradle.api.attributes.AttributeContainer;
+import org.gradle.api.attributes.HasConfigurableAttributes;
 
 import javax.annotation.Nullable;
 import java.util.Map;
@@ -32,7 +34,7 @@ import static groovy.lang.Closure.DELEGATE_FIRST;
  * <p>
  * For examples on configuring the exclude rules please refer to {@link #exclude(java.util.Map)}.
  */
-public interface ModuleDependency extends DirectDependency {
+public interface ModuleDependency extends Dependency, HasConfigurableAttributes<ModuleDependency> {
     /**
      * Adds an exclude rule to exclude transitive dependencies of this dependency.
      * <p>
@@ -156,4 +158,25 @@ public interface ModuleDependency extends DirectDependency {
      */
     ModuleDependency copy();
 
+    /**
+     * Returns the attributes for this dependency. Mutation of the attributes of a dependency must be done through
+     * the {@link #attributes(Action)} method.
+     *
+     * @return the attributes container for this dependency
+     *
+     * @since 4.8
+     */
+    @Incubating
+    AttributeContainer getAttributes();
+
+    /**
+     * Mutates the attributes of this dependency. Attributes are used during dependency resolution to select the appropriate
+     * target variant, in particular when a single component provides different variants.
+     *
+     * @param configureAction the attributes mutation action
+     *
+     * @since 4.8
+     */
+    @Incubating
+    ModuleDependency attributes(Action<? super AttributeContainer> configureAction);
 }
