@@ -17,9 +17,9 @@
 package org.gradle.api.tasks;
 
 import org.gradle.api.Action;
+import org.gradle.api.NamedDomainObjectProvider;
 import org.gradle.api.Incubating;
 import org.gradle.api.Task;
-import org.gradle.api.provider.Provider;
 
 /**
  * Providers a task of the given type.
@@ -28,7 +28,7 @@ import org.gradle.api.provider.Provider;
  * @since 4.8
  */
 @Incubating
-public interface TaskProvider<T extends Task> extends Provider<T> {
+public interface TaskProvider<T extends Task> extends NamedDomainObjectProvider<T> {
     /**
      * Configures the task with the given action. Actions are run in the order added.
      *
