@@ -212,6 +212,11 @@ public class CompositeDomainObjectSet<T> extends DelegatingDomainObjectSet<T> im
             throw new UnsupportedOperationException();
         }
 
+        @Override
+        public boolean addRealized(T element) {
+            throw new UnsupportedOperationException();
+        }
+
         @Override
         public boolean remove(Object o) {
             throw new UnsupportedOperationException();
@@ -272,17 +277,22 @@ public class CompositeDomainObjectSet<T> extends DelegatingDomainObjectSet<T> im
         }
 
         @Override
-        public void addPending(ProviderInternal<? extends T> provider) {
+        public boolean addPending(ProviderInternal<? extends T> provider) {
             throw new UnsupportedOperationException();
         }
 
         @Override
-        public void removePending(ProviderInternal<? extends T> provider) {
+        public boolean removePending(ProviderInternal<? extends T> provider) {
             throw new UnsupportedOperationException();
         }
 
         @Override
-        public void onRealize(Action<ProviderInternal<? extends T>> action) {
+        public void onRealize(Action<T> action) {
+
+        }
+
+        @Override
+        public void realizeExternal(ProviderInternal<? extends T> provider) {
 
         }
     }
