@@ -16,10 +16,13 @@
 
 package org.gradle.api.internal;
 
-import org.gradle.api.internal.changedetection.state.FileCollectionSnapshot;
-import org.gradle.api.internal.changedetection.state.NormalizedFileSnapshot;
 import org.gradle.internal.file.FileType;
+import org.gradle.internal.fingerprint.FileCollectionFingerprint;
+import org.gradle.internal.fingerprint.HistoricalFileCollectionFingerprint;
+import org.gradle.internal.fingerprint.NormalizedFileSnapshot;
+import org.gradle.internal.hash.HashCode;
 
+import javax.annotation.Nullable;
 import java.util.Map;
 
 public class OverlappingOutputs {
@@ -31,32 +34,35 @@ public class OverlappingOutputs {
         this.overlappedFilePath = overlappedFilePath;
     }
 
-    public static OverlappingOutputs detect(String propertyName, FileCollectionSnapshot previousExecution, FileCollectionSnapshot beforeExecution) {
+    @Nullable
+    public static OverlappingOutputs detect(String propertyName, HistoricalFileCollectionFingerprint previousExecution, FileCollectionFingerprint beforeExecution) {
         Map<String, NormalizedFileSnapshot> previousSnapshots = previousExecution.getSnapshots();
         Map<String, NormalizedFileSnapshot> beforeSnapshots = beforeExecution.getSnapshots();
 
-        for (Map.Entry<String, NormalizedFileSnapshot> beforeSnapshot : beforeSnapshots.entrySet()) {
-            String path = beforeSnapshot.getKey();
-            NormalizedFileSnapshot fileSnapshot = beforeSnapshot.getValue();
+        for (Map.Entry<String, NormalizedFileSnapshot> beforeEntry : beforeSnapshots.entrySet()) {
+            String path = beforeEntry.getKey();
+            NormalizedFileSnapshot beforeSnapshot = beforeEntry.getValue();
+            HashCode contentHash = beforeSnapshot.getNormalizedContentHash();
             NormalizedFileSnapshot previousSnapshot = previousSnapshots.get(path);
+            HashCode previousContentHash = previousSnapshot == null ? null : previousSnapshot.getNormalizedContentHash();
             // Missing files can be ignored
-            if (fileSnapshot.getSnapshot().getType() != FileType.Missing) {
-                if (createdSincePreviousExecution(previousSnapshot) || changedSincePreviousExecution(fileSnapshot, previousSnapshot)) {
-                    return new OverlappingOutputs(propertyName, fileSnapshot.getNormalizedPath());
+            if (beforeSnapshot.getType() != FileType.Missing) {
+                if (createdSincePreviousExecution(previousContentHash) || changedSincePreviousExecution(contentHash, previousContentHash)) {
+                    return new OverlappingOutputs(propertyName, path);
                 }
             }
         }
         return null;
     }
 
-    private static boolean changedSincePreviousExecution(NormalizedFileSnapshot fileSnapshot, NormalizedFileSnapshot previousSnapshot) {
+    private static boolean changedSincePreviousExecution(HashCode contentHash, HashCode previousContentHash) {
         // _changed_ since last execution, possibly by another task
-        return !previousSnapshot.getSnapshot().isContentUpToDate(fileSnapshot.getSnapshot());
+        return !contentHash.equals(previousContentHash);
     }
 
-    private static boolean createdSincePreviousExecution(NormalizedFileSnapshot previousSnapshot) {
+    private static boolean createdSincePreviousExecution(@Nullable HashCode previousContentHash) {
         // created since last execution, possibly by another task
-        return previousSnapshot == null;
+        return previousContentHash == null;
     }
 
     public String getPropertyName() {
