@@ -16,15 +16,21 @@
 package org.gradle.api.internal.collections;
 
 import org.gradle.api.Action;
+import org.gradle.internal.ImmutableActionSet;
 
 import javax.annotation.Nullable;
 
 public interface CollectionEventRegister<T> {
     boolean isSubscribed(@Nullable Class<?> type);
 
-    Action<T> getAddAction();
+    /**
+     * Returns a snapshot of the <em>current</em> set of actions to run when an element is added.
+     */
+    ImmutableActionSet<T> getAddActions();
 
-    Action<T> getRemoveAction();
+    void fireObjectAdded(T element);
+
+    void fireObjectRemoved(T element);
 
     void registerEagerAddAction(Class<? extends T> type, Action<? super T> addAction);
 
