@@ -35,7 +35,7 @@ abstract class AbstractArchiver implements CompressedReadableResource {
 
     abstract protected String getSchemePrefix();
 
-    abstract public InputStream read();
+    public abstract InputStream read();
 
     public String getDisplayName() {
         return resource.getDisplayName();
