@@ -30,7 +30,6 @@ import org.gradle.api.specs.Spec;
 import org.gradle.api.tasks.util.PatternFilterable;
 import org.gradle.api.tasks.util.PatternSet;
 import org.gradle.internal.Factory;
-import org.gradle.internal.FileUtils;
 import org.gradle.internal.nativeintegration.filesystem.FileSystem;
 import org.gradle.internal.nativeintegration.services.FileSystems;
 import org.gradle.util.GUtil;
@@ -59,24 +58,8 @@ public class DirectoryFileTree implements MinimalFileTree, PatternFilterableFile
     private final FileSystem fileSystem;
     private final Factory<DirectoryWalker> directoryWalkerFactory;
 
-    /**
-     * Use {@link DirectoryFileTreeFactory} instead.
-     */
-    @Deprecated
-    public DirectoryFileTree(File dir) {
-        this(dir, new PatternSet(), FileSystems.getDefault());
-    }
-
-    /**
-     * Use {@link DirectoryFileTreeFactory} instead.
-     */
-    @Deprecated
-    public DirectoryFileTree(File dir, PatternSet patternSet) {
-        this(dir, patternSet, FileSystems.getDefault());
-    }
-
     public DirectoryFileTree(File dir, PatternSet patternSet, FileSystem fileSystem) {
-        this(FileUtils.canonicalize(dir), patternSet, DEFAULT_DIRECTORY_WALKER_FACTORY, fileSystem, false);
+        this(dir, patternSet, DEFAULT_DIRECTORY_WALKER_FACTORY, fileSystem, false);
     }
 
     DirectoryFileTree(File dir, PatternSet patternSet, Factory<DirectoryWalker> directoryWalkerFactory, FileSystem fileSystem, boolean postfix) {
