@@ -16,9 +16,10 @@
 
 package org.gradle.api.internal.plugins;
 
-import org.gradle.api.Nullable;
 import org.gradle.plugin.use.PluginId;
 
+import javax.annotation.Nullable;
+
 public interface PluginImplementation<T> extends PotentialPlugin<T> {
     String getDisplayName();
 
