@@ -40,13 +40,24 @@ public abstract class AbstractMappingProvider<OUT, IN> extends AbstractProvider<
         return provider.isPresent();
     }
 
+    @Override
+    public OUT get() {
+        return map(provider.get());
+    }
+
     @Override
     public OUT getOrNull() {
-        if (provider.isPresent()) {
-            return map(provider.get());
+        IN value = provider.getOrNull();
+        if (value != null) {
+            return map(value);
         }
         return null;
     }
 
     protected abstract OUT map(IN v);
+
+    @Override
+    public String toString() {
+        return "transform(" + provider + ")";
+    }
 }
