@@ -16,18 +16,29 @@
 
 package org.gradle.api.internal.tasks;
 
+import org.gradle.api.Action;
 import org.gradle.api.NonNullApi;
 import org.gradle.api.Task;
+import org.gradle.api.internal.artifacts.transform.ArtifactTransformDependency;
 
 import javax.annotation.Nullable;
-import java.util.Collections;
+import java.util.Arrays;
 import java.util.Set;
 
 @NonNullApi
 public abstract class AbstractTaskDependency implements TaskDependencyInternal {
+    private static final WorkDependencyResolver<Task> IGNORE_ARTIFACT_TRANSFORM_RESOLVER = new WorkDependencyResolver<Task>() {
+        @Override
+        public boolean resolve(Task task, Object node, Action<? super Task> resolveAction) {
+            // Ignore artifact transforms
+            return node instanceof ArtifactTransformDependency;
+        }
+    };
+
+    @Override
     public Set<? extends Task> getDependencies(@Nullable Task task) {
         CachingTaskDependencyResolveContext<Task> context = new CachingTaskDependencyResolveContext<Task>(
-            Collections.singleton(WorkDependencyResolver.TASK_AS_TASK));
+            Arrays.asList(WorkDependencyResolver.TASK_AS_TASK, IGNORE_ARTIFACT_TRANSFORM_RESOLVER));
         return context.getDependencies(task, this);
     }
 }
