@@ -18,7 +18,7 @@ package org.gradle.api.internal.tasks;
 
 import org.gradle.api.NonNullApi;
 import org.gradle.api.file.FileCollection;
-import org.gradle.api.internal.file.FileResolver;
+import org.gradle.internal.file.PathToFileResolver;
 import org.gradle.util.DeferredUtil;
 
 import java.io.File;
@@ -27,11 +27,11 @@ import java.io.File;
 public class DefaultCacheableTaskOutputFilePropertySpec extends AbstractTaskOutputPropertySpec implements CacheableTaskOutputFilePropertySpec, DeclaredTaskOutputFileProperty {
     private final TaskPropertyFileCollection files;
     private final OutputType outputType;
-    private final FileResolver resolver;
+    private final PathToFileResolver resolver;
     private final ValidatingValue value;
     private final ValidationAction validationAction;
 
-    public DefaultCacheableTaskOutputFilePropertySpec(String taskName, FileResolver resolver, OutputType outputType, ValidatingValue path, ValidationAction validationAction) {
+    public DefaultCacheableTaskOutputFilePropertySpec(String taskName, PathToFileResolver resolver, OutputType outputType, ValidatingValue path, ValidationAction validationAction) {
         this.resolver = resolver;
         this.outputType = outputType;
         this.value = path;
