@@ -23,6 +23,7 @@ import org.gradle.internal.resources.ResourceLockState;
 import org.gradle.internal.work.WorkerLeaseRegistry;
 
 import javax.annotation.Nullable;
+import java.util.Collection;
 import java.util.Set;
 
 /**
@@ -30,9 +31,9 @@ import java.util.Set;
  */
 public interface TaskExecutionPlan extends Describable {
     @Nullable
-    TaskInfo selectNextTask(WorkerLeaseRegistry.WorkerLease workerLease, ResourceLockState resourceLockState);
+    WorkInfo selectNext(WorkerLeaseRegistry.WorkerLease workerLease, ResourceLockState resourceLockState);
 
-    void taskComplete(TaskInfo taskInfo);
+    void workComplete(WorkInfo workInfo);
 
     void abortAllAndFail(Throwable t);
 
@@ -59,10 +60,17 @@ public interface TaskExecutionPlan extends Describable {
      */
     Set<Task> getFilteredTasks();
 
-    void rethrowFailures();
+    /**
+     * Collects the current set of task failures into the given collection.
+     */
+    void collectFailures(Collection<? super Throwable> failures);
 
     boolean allTasksComplete();
 
     boolean hasWorkRemaining();
 
+    /**
+     * Returns the number of work items in the plan.
+     */
+    int size();
 }
