@@ -31,6 +31,7 @@ import org.gradle.api.internal.file.DefaultFileOperations;
 import org.gradle.api.internal.file.FileLookup;
 import org.gradle.api.internal.file.FileOperations;
 import org.gradle.api.internal.file.FileResolver;
+import org.gradle.api.internal.file.HasFileOperations;
 import org.gradle.api.internal.file.collections.DirectoryFileTreeFactory;
 import org.gradle.api.internal.initialization.ClassLoaderScope;
 import org.gradle.api.internal.initialization.ScriptHandlerFactory;
@@ -83,8 +84,8 @@ public abstract class DefaultScript extends BasicScript {
         StreamHasher streamHasher = services.get(StreamHasher.class);
         FileHasher fileHasher = services.get(FileHasher.class);
         TextResourceLoader textResourceLoader = services.get(TextResourceLoader.class);
-        if (target instanceof FileOperations) {
-            fileOperations = (FileOperations) target;
+        if (target instanceof HasFileOperations) {
+            fileOperations = ((HasFileOperations) target).getFileOperations();
         } else {
             File sourceFile = getScriptSource().getResource().getLocation().getFile();
             if (sourceFile != null) {
@@ -98,11 +99,15 @@ public abstract class DefaultScript extends BasicScript {
         providerFactory = services.get(ProviderFactory.class);
     }
 
-    @Override
     public FileResolver getFileResolver() {
         return fileOperations.getFileResolver();
     }
 
+    @Override
+    public FileOperations getFileOperations() {
+        return fileOperations;
+    }
+
     private DefaultObjectConfigurationAction createObjectConfigurationAction() {
         ClassLoaderScope classLoaderScope = __scriptServices.get(ClassLoaderScope.class);
         return new DefaultObjectConfigurationAction(
@@ -156,12 +161,12 @@ public abstract class DefaultScript extends BasicScript {
 
     @Override
     public ConfigurableFileCollection files(Object... paths) {
-        return fileOperations.files(paths);
+        return fileOperations.configurableFiles(paths);
     }
 
     @Override
     public ConfigurableFileCollection files(Object paths, Closure configureClosure) {
-        return ConfigureUtil.configure(configureClosure, fileOperations.files(paths));
+        return ConfigureUtil.configure(configureClosure, fileOperations.configurableFiles(paths));
     }
 
     @Override
@@ -204,12 +209,10 @@ public abstract class DefaultScript extends BasicScript {
         return copy(ConfigureUtil.configureUsing(closure));
     }
 
-    @Override
     public WorkResult copy(Action<? super CopySpec> action) {
         return fileOperations.copy(action);
     }
 
-    @Override
     public WorkResult sync(Action<? super CopySpec> action) {
         return fileOperations.sync(action);
     }
@@ -219,7 +222,6 @@ public abstract class DefaultScript extends BasicScript {
         return Actions.with(copySpec(), ConfigureUtil.configureUsing(closure));
     }
 
-    @Override
     public CopySpec copySpec() {
         return fileOperations.copySpec();
     }
@@ -234,7 +236,6 @@ public abstract class DefaultScript extends BasicScript {
         return fileOperations.delete(paths);
     }
 
-    @Override
     public WorkResult delete(Action<? super DeleteSpec> action) {
         return fileOperations.delete(action);
     }
