@@ -32,6 +32,7 @@ import org.gradle.execution.BuildConfigurationActionExecuter;
 import org.gradle.execution.BuildExecuter;
 import org.gradle.execution.MultipleBuildFailures;
 import org.gradle.execution.TaskExecutionGraphInternal;
+import org.gradle.internal.build.PublicBuildPath;
 import org.gradle.internal.concurrent.CompositeStoppable;
 import org.gradle.internal.operations.BuildOperationCategory;
 import org.gradle.internal.operations.BuildOperationContext;
@@ -73,7 +74,8 @@ public class DefaultGradleLauncher implements GradleLauncher {
     private final BuildScopeServices buildServices;
     private final List<?> servicesToStop;
     private final IncludedBuildControllers includedBuildControllers;
-    private GradleInternal gradle;
+    private final PublicBuildPath fromBuild;
+    private final GradleInternal gradle;
     private SettingsInternal settings;
     private Stage stage;
 
@@ -82,7 +84,7 @@ public class DefaultGradleLauncher implements GradleLauncher {
                                  BuildListener buildListener, ModelConfigurationListener modelConfigurationListener,
                                  BuildCompletionListener buildCompletionListener, BuildOperationExecutor operationExecutor,
                                  BuildConfigurationActionExecuter buildConfigurationActionExecuter, BuildExecuter buildExecuter,
-                                 BuildScopeServices buildServices, List<?> servicesToStop, IncludedBuildControllers includedBuildControllers) {
+                                 BuildScopeServices buildServices, List<?> servicesToStop, IncludedBuildControllers includedBuildControllers, PublicBuildPath fromBuild) {
         this.gradle = gradle;
         this.initScriptHandler = initScriptHandler;
         this.settingsLoader = settingsLoader;
@@ -98,6 +100,7 @@ public class DefaultGradleLauncher implements GradleLauncher {
         this.buildServices = buildServices;
         this.servicesToStop = servicesToStop;
         this.includedBuildControllers = includedBuildControllers;
+        this.fromBuild = fromBuild;
     }
 
     @Override
@@ -255,6 +258,11 @@ public class DefaultGradleLauncher implements GradleLauncher {
                     public String getBuildPath() {
                         return gradle.getIdentityPath().toString();
                     }
+
+                    @Override
+                    public String getIncludedBy() {
+                        return fromBuild == null ? null : fromBuild.getBuildPath().toString();
+                    }
                 });
         }
     }
@@ -283,11 +291,11 @@ public class DefaultGradleLauncher implements GradleLauncher {
             }
             builder.totalProgress(settings.getProjectRegistry().size());
             return builder.details(new ConfigureBuildBuildOperationType.Details() {
-                    @Override
-                    public String getBuildPath() {
-                        return getGradle().getIdentityPath().toString();
-                    }
-                });
+                @Override
+                public String getBuildPath() {
+                    return getGradle().getIdentityPath().toString();
+                }
+            });
         }
     }
 
