@@ -19,7 +19,6 @@ package org.gradle.util;
 import groovy.lang.Closure;
 import org.codehaus.groovy.runtime.GeneratedClosure;
 import org.gradle.api.Action;
-import org.gradle.api.Nullable;
 import org.gradle.api.internal.ClosureBackedAction;
 import org.gradle.api.internal.DynamicObjectUtil;
 import org.gradle.internal.Actions;
@@ -27,6 +26,7 @@ import org.gradle.internal.metaobject.ConfigureDelegate;
 import org.gradle.internal.metaobject.DynamicInvokeResult;
 import org.gradle.internal.metaobject.DynamicObject;
 
+import javax.annotation.Nullable;
 import java.util.Collection;
 import java.util.Map;
 
@@ -117,12 +117,7 @@ public class ConfigureUtil {
             return Actions.doNothing();
         }
 
-        return new Action<T>() {
-            @Override
-            public void execute(T t) {
-                configure(configureClosure, t);
-            }
-        };
+        return new WrappedConfigureAction<T>(configureClosure);
     }
 
     /**
@@ -159,4 +154,21 @@ public class ConfigureUtil {
         Closure withNewOwner = configureClosure.rehydrate(target, closureDelegate, configureClosure.getThisObject());
         new ClosureBackedAction<T>(withNewOwner, Closure.OWNER_ONLY, false).execute(target);
     }
+
+    public static class WrappedConfigureAction<T> implements Action<T> {
+        private final Closure configureClosure;
+
+        WrappedConfigureAction(Closure configureClosure) {
+            this.configureClosure = configureClosure;
+        }
+
+        @Override
+        public void execute(T t) {
+            configure(configureClosure, t);
+        }
+
+        public Closure getConfigureClosure() {
+            return configureClosure;
+        }
+    }
 }
