@@ -17,8 +17,9 @@ package org.gradle.api.internal.artifacts;
 
 import org.gradle.api.internal.artifacts.ivyservice.ivyresolve.strategy.VersionSelector;
 
-public interface ResolvedVersionConstraint extends ImmutableVersionConstraint {
+public interface ResolvedVersionConstraint {
     VersionSelector getPreferredSelector();
     VersionSelector getRejectedSelector();
     boolean isRejectAll();
+    boolean isPrefer();
 }
