@@ -173,6 +173,11 @@ public class ClientModuleResolver implements ComponentMetaDataResolver {
             return delegate.getStatusScheme();
         }
 
+        @Override
+        public ImmutableList<? extends ComponentIdentifier> getPlatformOwners() {
+            return ImmutableList.of();
+        }
+
         @Override
         public AttributeContainer getAttributes() {
             return delegate.getAttributes();
