@@ -18,7 +18,7 @@ package org.gradle.api.internal.artifacts.ivyservice.ivyresolve.strategy;
 /**
  * Version matcher for dynamic version selectors ending in '+'.
  */
-public class SubVersionSelector extends AbstractStringVersionSelector {
+public class SubVersionSelector extends AbstractStringVersionSelector implements LockingAwareSelector {
     private final String prefix;
 
     public SubVersionSelector(String selector) {
@@ -45,4 +45,25 @@ public class SubVersionSelector extends AbstractStringVersionSelector {
     public boolean accept(String candidate) {
         return candidate.startsWith(prefix);
     }
+
+    @Override
+    public boolean canShortCircuitWhenVersionAlreadyPreselected() {
+        return false;
     }
+
+    @Override
+    public VersionSelector forLocking() {
+        return new LockingAwareSubVersionSelector(getSelector());
+    }
+
+    private static class LockingAwareSubVersionSelector extends SubVersionSelector {
+        public LockingAwareSubVersionSelector(String selector) {
+            super(selector);
+        }
+
+        @Override
+        public boolean canShortCircuitWhenVersionAlreadyPreselected() {
+            return true;
+        }
+    }
+}
