@@ -52,7 +52,8 @@ import java.util.concurrent.TimeUnit;
 import static org.gradle.api.internal.artifacts.configurations.MutationValidator.MutationType.STRATEGY;
 
 public class DefaultResolutionStrategy implements ResolutionStrategyInternal {
-    private final Set<ModuleVersionSelector> forcedModules = new LinkedHashSet<ModuleVersionSelector>();
+    private final Set<Object> forcedModules = new LinkedHashSet<Object>();
+    private Set<ModuleVersionSelector> parsedForcedModules;
     private ConflictResolution conflictResolution = ConflictResolution.latest;
     private final DefaultComponentSelectionRules componentSelectionRules;
 
@@ -96,7 +97,10 @@ public class DefaultResolutionStrategy implements ResolutionStrategyInternal {
     }
 
     public Set<ModuleVersionSelector> getForcedModules() {
-        return Collections.unmodifiableSet(forcedModules);
+        if (parsedForcedModules == null) {
+            parsedForcedModules = ModuleVersionSelectorParsers.multiParser().parseNotation(forcedModules);
+        }
+        return Collections.unmodifiableSet(parsedForcedModules);
     }
 
     public ResolutionStrategy failOnVersionConflict() {
@@ -132,8 +136,8 @@ public class DefaultResolutionStrategy implements ResolutionStrategyInternal {
 
     public DefaultResolutionStrategy force(Object... moduleVersionSelectorNotations) {
         mutationValidator.validateMutation(STRATEGY);
-        Set<ModuleVersionSelector> modules = ModuleVersionSelectorParsers.multiParser().parseNotation(moduleVersionSelectorNotations);
-        this.forcedModules.addAll(modules);
+        parsedForcedModules = null;
+        Collections.addAll(forcedModules, moduleVersionSelectorNotations);
         return this;
     }
 
@@ -144,6 +148,7 @@ public class DefaultResolutionStrategy implements ResolutionStrategyInternal {
     }
 
     public Action<DependencySubstitution> getDependencySubstitutionRule() {
+        Set<ModuleVersionSelector> forcedModules = getForcedModules();
         Action<DependencySubstitution> moduleForcingResolveRule = Cast.uncheckedCast(forcedModules.isEmpty() ? Actions.doNothing() : new ModuleForcingResolveRule(forcedModules, moduleIdentifierFactory));
         Action<DependencySubstitution> localDependencySubstitutionsAction = this.dependencySubstitutions.getRuleAction();
         Action<DependencySubstitution> globalDependencySubstitutionRulesAction = globalDependencySubstitutionRules.getRuleAction();
@@ -159,12 +164,10 @@ public class DefaultResolutionStrategy implements ResolutionStrategyInternal {
         return assumeFluidDependencies || dependencySubstitutions.hasRules() || globalDependencySubstitutionRules.hasRules() || vcsResolver.hasRules();
     }
 
-
     public DefaultResolutionStrategy setForcedModules(Object ... moduleVersionSelectorNotations) {
         mutationValidator.validateMutation(STRATEGY);
-        Set<ModuleVersionSelector> modules = ModuleVersionSelectorParsers.multiParser().parseNotation(moduleVersionSelectorNotations);
         this.forcedModules.clear();
-        this.forcedModules.addAll(modules);
+        force(moduleVersionSelectorNotations);
         return this;
     }
 
@@ -216,7 +219,7 @@ public class DefaultResolutionStrategy implements ResolutionStrategyInternal {
         } else if (conflictResolution == ConflictResolution.preferProjectModules) {
             out.preferProjectModules();
         }
-        out.setForcedModules(getForcedModules());
+        out.setForcedModules(forcedModules);
         for (SpecRuleAction<? super ComponentSelection> ruleAction : componentSelectionRules.getRules()) {
             out.getComponentSelection().addRule(ruleAction);
         }
