@@ -27,6 +27,7 @@ import org.gradle.api.artifacts.ModuleVersionIdentifier;
 import org.gradle.api.artifacts.component.ComponentIdentifier;
 import org.gradle.api.attributes.AttributeContainer;
 import org.gradle.api.capabilities.Capability;
+import org.gradle.api.internal.artifacts.ivyservice.ivyresolve.RepositoryChainModuleSource;
 import org.gradle.api.internal.artifacts.ivyservice.ivyresolve.strategy.VersionSelector;
 import org.gradle.api.internal.artifacts.ivyservice.resolveengine.ComponentResolutionState;
 import org.gradle.api.internal.artifacts.ivyservice.resolveengine.graph.DependencyGraphComponent;
@@ -42,6 +43,7 @@ import org.gradle.internal.component.external.model.ImmutableCapability;
 import org.gradle.internal.component.model.ComponentOverrideMetadata;
 import org.gradle.internal.component.model.ComponentResolveMetadata;
 import org.gradle.internal.component.model.DefaultComponentOverrideMetadata;
+import org.gradle.internal.component.model.ModuleSource;
 import org.gradle.internal.resolve.ModuleVersionResolveException;
 import org.gradle.internal.resolve.RejectedByAttributesVersion;
 import org.gradle.internal.resolve.RejectedByRuleVersion;
@@ -111,6 +113,15 @@ public class ComponentState implements ComponentResolutionState, DependencyGraph
         return id;
     }
 
+    @Override
+    public String getRepositoryName() {
+        ModuleSource moduleSource = metadata.getSource();
+        if (moduleSource instanceof RepositoryChainModuleSource) {
+            return ((RepositoryChainModuleSource) moduleSource).getRepositoryName();
+        }
+        return null;
+    }
+
     @Override
     public ModuleVersionIdentifier getModuleVersion() {
         return id;
@@ -142,6 +153,11 @@ public class ComponentState implements ComponentResolutionState, DependencyGraph
         return metadata;
     }
 
+    ComponentResolveMetadata getMetadataWithoutRetryMissing() {
+        resolve(false);
+        return metadata;
+    }
+
     @Override
     public ComponentIdentifier getComponentId() {
         // Use the resolved component id if available: this ensures that Maven Snapshot ids are correctly reported
@@ -177,6 +193,10 @@ public class ComponentState implements ComponentResolutionState, DependencyGraph
     }
 
     public void resolve() {
+        resolve(true);
+    }
+
+    public void resolve(boolean retryMissing) {
         if (alreadyResolved()) {
             return;
         }
@@ -184,7 +204,7 @@ public class ComponentState implements ComponentResolutionState, DependencyGraph
         // Any metadata overrides (e.g classifier/artifacts/client-module) will be taken from the first dependency that referenced this component
         ComponentOverrideMetadata componentOverrideMetadata = DefaultComponentOverrideMetadata.forDependency(firstSelectedBy.getDependencyMetadata());
 
-        DefaultBuildableComponentResolveResult result = new DefaultBuildableComponentResolveResult();
+        DefaultBuildableComponentResolveResult result = new DefaultBuildableComponentResolveResult(retryMissing);
         resolver.resolve(componentIdentifier, componentOverrideMetadata, result);
         if (result.getFailure() != null) {
             metadataResolveFailure = result.getFailure();
