@@ -119,14 +119,26 @@ class EdgeState implements DependencyGraphEdge {
     }
 
     public void removeFromTargetConfigurations() {
-        for (NodeState targetConfiguration : targetNodes) {
-            targetConfiguration.removeIncomingEdge(this);
+        if (!targetNodes.isEmpty()) {
+            for (NodeState targetConfiguration : targetNodes) {
+                targetConfiguration.removeIncomingEdge(this);
+            }
+            targetNodes.clear();
         }
-        targetNodes.clear();
         targetNodeSelectionFailure = null;
     }
 
-    public void restart(ComponentState selected) {
+    /**
+     * Call this method to attach a failure late in the process. This is typically
+     * done when a failure is caused by graph validation. In that case we want to
+     * perform as much resolution as possible, still have a valid graph, but in the
+     * end fail resolution.
+     */
+    public void failWith(Throwable err) {
+        targetNodeSelectionFailure = new ModuleVersionResolveException(dependencyState.getRequested(), err);
+    }
+
+    public void restart() {
         if (from.isSelected()) {
             removeFromTargetConfigurations();
             attachToTargetConfigurations();
