@@ -16,7 +16,6 @@
 
 package org.gradle.api.internal.artifacts.ivyservice.resolveengine.graph.builder;
 
-import com.google.common.collect.Sets;
 import org.gradle.api.artifacts.ModuleIdentifier;
 import org.gradle.api.artifacts.ModuleVersionIdentifier;
 import org.gradle.api.artifacts.component.ComponentIdentifier;
@@ -25,6 +24,8 @@ import org.gradle.api.internal.artifacts.ComponentSelectorConverter;
 import org.gradle.api.internal.artifacts.ResolvedConfigurationIdentifier;
 import org.gradle.api.internal.artifacts.dsl.ModuleReplacementsData;
 import org.gradle.api.internal.artifacts.ivyservice.dependencysubstitution.DependencySubstitutionApplicator;
+import org.gradle.api.internal.artifacts.ivyservice.ivyresolve.strategy.Version;
+import org.gradle.api.internal.artifacts.ivyservice.ivyresolve.strategy.VersionParser;
 import org.gradle.api.internal.artifacts.ivyservice.ivyresolve.strategy.VersionSelectorScheme;
 import org.gradle.api.internal.artifacts.ivyservice.resolveengine.excludes.ModuleExclusions;
 import org.gradle.api.internal.artifacts.ivyservice.resolveengine.graph.selectors.ComponentStateFactory;
@@ -41,10 +42,10 @@ import org.gradle.internal.resolve.result.ComponentResolveResult;
 
 import java.util.ArrayDeque;
 import java.util.Collection;
+import java.util.Comparator;
 import java.util.Deque;
 import java.util.LinkedHashMap;
 import java.util.Map;
-import java.util.Set;
 
 /**
  * Global resolution state.
@@ -58,7 +59,6 @@ class ResolveState implements ComponentStateFactory<ComponentState> {
     private final IdGenerator<Long> idGenerator;
     private final DependencyToComponentIdResolver idResolver;
     private final ComponentMetaDataResolver metaDataResolver;
-    private final Set<NodeState> queued = Sets.newHashSet();
     private final Deque<NodeState> queue = new ArrayDeque<NodeState>();
     private final AttributesSchemaInternal attributesSchema;
     private final ModuleExclusions moduleExclusions;
@@ -70,12 +70,15 @@ class ResolveState implements ComponentStateFactory<ComponentState> {
     private final DependencySubstitutionApplicator dependencySubstitutionApplicator;
     private final VariantNameBuilder variantNameBuilder = new VariantNameBuilder();
     private final VersionSelectorScheme versionSelectorScheme;
+    private final Comparator<Version> versionComparator;
+    private final VersionParser versionParser;
 
     public ResolveState(IdGenerator<Long> idGenerator, ComponentResolveResult rootResult, String rootConfigurationName, DependencyToComponentIdResolver idResolver,
                         ComponentMetaDataResolver metaDataResolver, Spec<? super DependencyMetadata> edgeFilter, AttributesSchemaInternal attributesSchema,
                         ModuleExclusions moduleExclusions, ModuleReplacementsData moduleReplacementsData,
                         ComponentSelectorConverter componentSelectorConverter, ImmutableAttributesFactory attributesFactory,
-                        DependencySubstitutionApplicator dependencySubstitutionApplicator, VersionSelectorScheme versionSelectorScheme) {
+                        DependencySubstitutionApplicator dependencySubstitutionApplicator, VersionSelectorScheme versionSelectorScheme,
+                        Comparator<Version> versionComparator, VersionParser versionParser) {
         this.idGenerator = idGenerator;
         this.idResolver = idResolver;
         this.metaDataResolver = metaDataResolver;
@@ -87,6 +90,8 @@ class ResolveState implements ComponentStateFactory<ComponentState> {
         this.attributesFactory = attributesFactory;
         this.dependencySubstitutionApplicator = dependencySubstitutionApplicator;
         this.versionSelectorScheme = versionSelectorScheme;
+        this.versionComparator = versionComparator;
+        this.versionParser = versionParser;
         ComponentState rootVersion = getRevision(rootResult.getId(), rootResult.getModuleVersionId(), rootResult.getMetadata());
         final ResolvedConfigurationIdentifier id = new ResolvedConfigurationIdentifier(rootVersion.getId(), rootConfigurationName);
         ConfigurationMetadata configurationMetadata = rootVersion.getMetadata().getConfiguration(id.getConfiguration());
@@ -112,7 +117,7 @@ class ResolveState implements ComponentStateFactory<ComponentState> {
     public ModuleResolveState getModule(ModuleIdentifier id) {
         ModuleResolveState module = modules.get(id);
         if (module == null) {
-            module = new ModuleResolveState(idGenerator, id, metaDataResolver, variantNameBuilder, attributesFactory);
+            module = new ModuleResolveState(idGenerator, id, metaDataResolver, variantNameBuilder, attributesFactory, versionComparator, versionParser);
             modules.put(id, module);
         }
         return module;
@@ -166,8 +171,7 @@ class ResolveState implements ComponentStateFactory<ComponentState> {
 
     public NodeState pop() {
         NodeState next = queue.removeFirst();
-        queued.remove(next);
-        return next;
+        return next.dequeue();
     }
 
     /**
@@ -176,7 +180,7 @@ class ResolveState implements ComponentStateFactory<ComponentState> {
     public void onMoreSelected(NodeState node) {
         // Add to the end of the queue, so that we traverse the graph in breadth-wise order to pick up as many conflicts as
         // possible before attempting to resolve them
-        if (queued.add(node)) {
+        if (node.enqueue()) {
             queue.addLast(node);
         }
     }
@@ -186,7 +190,7 @@ class ResolveState implements ComponentStateFactory<ComponentState> {
      */
     public void onFewerSelected(NodeState node) {
         // Add to the front of the queue, to flush out configurations that are no longer required.
-        if (queued.add(node)) {
+        if (node.enqueue()) {
             queue.addFirst(node);
         }
     }
