@@ -17,7 +17,6 @@ package org.gradle.api.internal.artifacts.ivyservice.resolveengine.graph.selecto
 
 import com.google.common.collect.Lists;
 import org.gradle.api.artifacts.ModuleIdentifier;
-import org.gradle.api.artifacts.ModuleVersionIdentifier;
 import org.gradle.api.internal.artifacts.ResolvedVersionConstraint;
 import org.gradle.api.internal.artifacts.ivyservice.ivyresolve.strategy.UnionVersionSelector;
 import org.gradle.api.internal.artifacts.ivyservice.ivyresolve.strategy.VersionSelector;
@@ -61,7 +60,7 @@ public class SelectorStateResolver<T extends ComponentResolutionState> {
         }
 
         // If we have a single common resolution, no conflicts to resolve
-        if (hasNoConflicts(candidates)) {
+        if (candidates.size() == 1) {
             return candidates.get(0);
         }
 
@@ -69,25 +68,6 @@ public class SelectorStateResolver<T extends ComponentResolutionState> {
         return resolveConflicts(candidates);
     }
 
-    private boolean hasNoConflicts(List<T> candidates) {
-        if (candidates.size() == 1) {
-            return true;
-        }
-
-        // If all versions are the same, there is no conflict
-        ModuleVersionIdentifier id = null;
-        for (T candidate : candidates) {
-            if (id == null) {
-                id = candidate.getId();
-            } else {
-                if (!id.equals(candidate.getId())) {
-                    return false;
-                }
-            }
-        }
-        return true;
-    }
-
     private List<T> resolveSelectors(List<? extends ResolvableSelectorState> selectors, VersionSelector allRejects) {
         if (selectors.size() == 1) {
             ResolvableSelectorState selectorState = selectors.get(0);
