@@ -21,14 +21,18 @@ import org.gradle.api.Action;
 import org.gradle.api.internal.artifacts.ivyservice.ArtifactCacheMetadata;
 import org.gradle.api.internal.changedetection.state.FileSystemSnapshotter;
 import org.gradle.api.internal.changedetection.state.InMemoryCacheDecoratorFactory;
-import org.gradle.api.internal.changedetection.state.Snapshot;
+import org.gradle.api.internal.changedetection.state.mirror.PhysicalSnapshot;
 import org.gradle.cache.CacheBuilder;
 import org.gradle.cache.CacheRepository;
+import org.gradle.cache.CleanupAction;
 import org.gradle.cache.FileLockManager;
 import org.gradle.cache.PersistentCache;
 import org.gradle.cache.PersistentIndexedCache;
 import org.gradle.cache.PersistentIndexedCacheParameters;
+import org.gradle.cache.internal.CompositeCleanupAction;
+import org.gradle.cache.internal.LeastRecentlyUsedCacheCleanup;
 import org.gradle.cache.internal.ProducerGuard;
+import org.gradle.cache.internal.SingleDepthFilesFinder;
 import org.gradle.caching.internal.DefaultBuildCacheHasher;
 import org.gradle.initialization.RootBuildLifecycleListener;
 import org.gradle.internal.Factory;
@@ -36,8 +40,11 @@ import org.gradle.internal.UncheckedException;
 import org.gradle.internal.concurrent.Stoppable;
 import org.gradle.internal.hash.HashCode;
 import org.gradle.internal.resource.local.DefaultPathKeyFileStore;
+import org.gradle.internal.resource.local.FileAccessTimeJournal;
+import org.gradle.internal.resource.local.FileAccessTracker;
 import org.gradle.internal.resource.local.FileStore;
 import org.gradle.internal.resource.local.FileStoreAddActionException;
+import org.gradle.internal.resource.local.SingleDepthFileAccessTracker;
 import org.gradle.internal.serialize.BaseSerializerFactory;
 import org.gradle.internal.serialize.HashCodeSerializer;
 import org.gradle.internal.serialize.ListSerializer;
@@ -50,31 +57,44 @@ import java.util.concurrent.ConcurrentHashMap;
 
 import static org.gradle.api.internal.artifacts.ivyservice.CacheLayout.TRANSFORMS_META_DATA;
 import static org.gradle.api.internal.artifacts.ivyservice.CacheLayout.TRANSFORMS_STORE;
+import static org.gradle.cache.internal.LeastRecentlyUsedCacheCleanup.DEFAULT_MAX_AGE_IN_DAYS_FOR_RECREATABLE_CACHE_ENTRIES;
 import static org.gradle.cache.internal.filelock.LockOptionsBuilder.mode;
 
 public class DefaultTransformedFileCache implements TransformedFileCache, Stoppable, RootBuildLifecycleListener {
+
+    private static final int FILE_TREE_DEPTH_TO_TRACK_AND_CLEANUP = 2;
+    private static final String CACHE_PREFIX = TRANSFORMS_META_DATA.getKey() + "/";
+
     private final PersistentCache cache;
     private final PersistentIndexedCache<HashCode, List<File>> indexedCache;
     private final FileStore<String> fileStore;
     private final ProducerGuard<CacheKey> producing = ProducerGuard.adaptive();
     private final Map<CacheKey, List<File>> resultHashToResult = new ConcurrentHashMap<CacheKey, List<File>>();
     private final FileSystemSnapshotter fileSystemSnapshotter;
+    private final FileAccessTracker fileAccessTracker;
 
-    public DefaultTransformedFileCache(ArtifactCacheMetadata artifactCacheMetadata, CacheRepository cacheRepository, InMemoryCacheDecoratorFactory cacheDecoratorFactory, FileSystemSnapshotter fileSystemSnapshotter) {
+    public DefaultTransformedFileCache(ArtifactCacheMetadata artifactCacheMetadata, CacheRepository cacheRepository, InMemoryCacheDecoratorFactory cacheDecoratorFactory,
+                                       FileSystemSnapshotter fileSystemSnapshotter, FileAccessTimeJournal fileAccessTimeJournal) {
         this.fileSystemSnapshotter = fileSystemSnapshotter;
         File transformsStoreDirectory = artifactCacheMetadata.getTransformsStoreDirectory();
         File filesOutputDirectory = new File(transformsStoreDirectory, TRANSFORMS_STORE.getKey());
         fileStore = new DefaultPathKeyFileStore(filesOutputDirectory);
         cache = cacheRepository
             .cache(transformsStoreDirectory)
+            .withCleanup(createCleanupAction(filesOutputDirectory, fileAccessTimeJournal))
             .withCrossVersionCache(CacheBuilder.LockTarget.DefaultTarget)
             .withDisplayName("Artifact transforms cache")
             .withLockOptions(mode(FileLockManager.LockMode.None)) // Lock on demand
             .open();
-        String cacheName = TRANSFORMS_META_DATA.getKey() + "/results";
-        PersistentIndexedCacheParameters<HashCode, List<File>> cacheParameters = new PersistentIndexedCacheParameters<HashCode, List<File>>(cacheName, new HashCodeSerializer(), new ListSerializer<File>(BaseSerializerFactory.FILE_SERIALIZER))
-            .cacheDecorator(cacheDecoratorFactory.decorator(1000, true));
-        indexedCache = cache.createCache(cacheParameters);
+        indexedCache = cache.createCache(PersistentIndexedCacheParameters.of(CACHE_PREFIX + "results", new HashCodeSerializer(), new ListSerializer<File>(BaseSerializerFactory.FILE_SERIALIZER))
+            .cacheDecorator(cacheDecoratorFactory.decorator(1000, true)));
+        fileAccessTracker = new SingleDepthFileAccessTracker(fileAccessTimeJournal, filesOutputDirectory, FILE_TREE_DEPTH_TO_TRACK_AND_CLEANUP);
+    }
+
+    private CleanupAction createCleanupAction(File filesOutputDirectory, FileAccessTimeJournal fileAccessTimeJournal) {
+        return CompositeCleanupAction.builder()
+            .add(filesOutputDirectory, new LeastRecentlyUsedCacheCleanup(new SingleDepthFilesFinder(FILE_TREE_DEPTH_TO_TRACK_AND_CLEANUP), fileAccessTimeJournal, DEFAULT_MAX_AGE_IN_DAYS_FOR_RECREATABLE_CACHE_ENTRIES))
+            .build();
     }
 
     @Override
@@ -98,7 +118,7 @@ public class DefaultTransformedFileCache implements TransformedFileCache, Stoppa
     }
 
     @Override
-    public List<File> getResult(final File inputFile, HashCode inputsHash, final BiFunction<List<File>, File, File> transformer) {
+    public List<File> getResult(File inputFile, HashCode inputsHash, BiFunction<List<File>, File, File> transformer) {
         final CacheKey resultHash = getCacheKey(inputFile, inputsHash);
         List<File> files = resultHashToResult.get(resultHash);
         if (files != null) {
@@ -150,6 +170,7 @@ public class DefaultTransformedFileCache implements TransformedFileCache, Stoppa
                     }
                 });
 
+                fileAccessTracker.markAccessed(files);
                 resultHashToResult.put(cacheKey, files);
                 return files;
             }
@@ -157,8 +178,8 @@ public class DefaultTransformedFileCache implements TransformedFileCache, Stoppa
     }
 
     private CacheKey getCacheKey(File inputFile, HashCode inputsHash) {
-        Snapshot inputFileSnapshot = fileSystemSnapshotter.snapshotAll(inputFile);
-        return new CacheKey(inputFileSnapshot, inputsHash);
+        PhysicalSnapshot snapshot = fileSystemSnapshotter.snapshot(inputFile);
+        return new CacheKey(inputsHash, snapshot.getAbsolutePath(), snapshot.getHash());
     }
 
     /**
@@ -167,19 +188,21 @@ public class DefaultTransformedFileCache implements TransformedFileCache, Stoppa
      * operation, so we only calculate it when we have a cache miss in memory.
      */
     private static class CacheKey {
-        private final Snapshot fileSnapshot;
+        private final String absolutePath;
+        private final HashCode fileContentHash;
         private final HashCode inputHash;
 
-        public CacheKey(Snapshot fileSnapshot, HashCode inputHash) {
-            this.fileSnapshot = fileSnapshot;
+        public CacheKey(HashCode inputHash, String absolutePath, HashCode fileContentHash) {
+            this.absolutePath = absolutePath;
+            this.fileContentHash = fileContentHash;
             this.inputHash = inputHash;
         }
 
-
         public HashCode getPersistentCacheKey() {
             DefaultBuildCacheHasher hasher = new DefaultBuildCacheHasher();
             hasher.putHash(inputHash);
-            fileSnapshot.appendToHasher(hasher);
+            hasher.putString(absolutePath);
+            hasher.putHash(fileContentHash);
             return hasher.hash();
         }
 
@@ -194,15 +217,19 @@ public class DefaultTransformedFileCache implements TransformedFileCache, Stoppa
 
             CacheKey cacheKey = (CacheKey) o;
 
-            if (!fileSnapshot.equals(cacheKey.fileSnapshot)) {
+            if (!fileContentHash.equals(cacheKey.fileContentHash)) {
                 return false;
             }
-            return inputHash.equals(cacheKey.inputHash);
+            if (!inputHash.equals(cacheKey.inputHash)) {
+                return false;
+            }
+            return absolutePath.equals(cacheKey.absolutePath);
         }
 
         @Override
         public int hashCode() {
-            int result = fileSnapshot.hashCode();
+            int result = fileContentHash.hashCode();
+            result = 31 * result + absolutePath.hashCode();
             result = 31 * result + inputHash.hashCode();
             return result;
         }
