@@ -18,8 +18,14 @@ package org.gradle.api.tasks.diagnostics.internal.graph.nodes;
 
 import org.gradle.api.artifacts.result.ResolvedVariantResult;
 
+import java.util.List;
 import java.util.Set;
 
+/**
+ * A renderable dependency may be a dependency OR something related
+ * to a dependency, like a header. In practice, for a single actual
+ * dependency, we may render multiple renderable dependencies.
+ */
 public interface RenderableDependency {
     Object getId();
     String getName();
@@ -27,6 +33,7 @@ public interface RenderableDependency {
     ResolvedVariantResult getResolvedVariant();
     ResolutionState getResolutionState();
     Set<? extends RenderableDependency> getChildren();
+    List<Section> getExtraDetails();
 
     enum ResolutionState {
         FAILED,
