@@ -33,6 +33,7 @@ import org.gradle.api.specs.Spec;
 import org.gradle.api.tasks.Delete;
 import org.gradle.api.tasks.Sync;
 import org.gradle.api.tasks.TaskContainer;
+import org.gradle.api.tasks.TaskProvider;
 import org.gradle.ide.xcode.XcodeExtension;
 import org.gradle.ide.xcode.XcodeRootExtension;
 import org.gradle.ide.xcode.internal.DefaultXcodeExtension;
@@ -48,6 +49,7 @@ import org.gradle.ide.xcode.tasks.GenerateSchemeFileTask;
 import org.gradle.ide.xcode.tasks.GenerateWorkspaceSettingsFileTask;
 import org.gradle.ide.xcode.tasks.GenerateXcodeProjectFileTask;
 import org.gradle.ide.xcode.tasks.GenerateXcodeWorkspaceFileTask;
+import org.gradle.internal.Actions;
 import org.gradle.language.cpp.CppBinary;
 import org.gradle.language.cpp.CppExecutable;
 import org.gradle.language.cpp.CppSharedLibrary;
@@ -99,14 +101,14 @@ public class XcodePlugin extends IdePlugin {
 
     @Override
     protected void onApply(final Project project) {
-        Task lifecycleTask = getLifecycleTask();
-        lifecycleTask.setDescription("Generates XCode project files (pbxproj, xcworkspace, xcscheme)");
+        TaskProvider<? extends Task> lifecycleTask = getLifecycleTask();
+        lifecycleTask.configure(withDescription("Generates XCode project files (pbxproj, xcworkspace, xcscheme)"));
 
         if (isRoot()) {
             DefaultXcodeRootExtension xcode = (DefaultXcodeRootExtension) project.getExtensions().create(XcodeRootExtension.class, "xcode", DefaultXcodeRootExtension.class, objectFactory);
             xcodeProject = xcode.getProject();
             final GenerateXcodeWorkspaceFileTask workspaceTask = createWorkspaceTask(project, xcode.getWorkspace());
-            lifecycleTask.dependsOn(workspaceTask);
+            lifecycleTask.configure(dependsOn(workspaceTask));
             addWorkspace(xcode.getWorkspace());
         } else {
             DefaultXcodeExtension xcode = (DefaultXcodeExtension) project.getExtensions().create(XcodeExtension.class, "xcode", DefaultXcodeExtension.class, objectFactory);
@@ -116,7 +118,7 @@ public class XcodePlugin extends IdePlugin {
         xcodeProject.setLocationDir(project.file(project.getName() + ".xcodeproj"));
 
         GenerateXcodeProjectFileTask projectTask = createProjectTask((ProjectInternal) project);
-        lifecycleTask.dependsOn(projectTask);
+        lifecycleTask.configure(dependsOn(projectTask));
 
         project.getTasks().addRule("Xcode bridge tasks begin with _xcode. Do not call these directly.", new XcodeBridge(xcodeProject, project));
 
@@ -135,13 +137,12 @@ public class XcodePlugin extends IdePlugin {
     }
 
     private void configureXcodeCleanTask(Project project) {
-        getCleanTask().setDescription("Cleans XCode project files (xcodeproj)");
         Delete cleanTask = project.getTasks().create("cleanXcodeProject", Delete.class);
         cleanTask.delete(xcodeProject.getLocationDir());
         if (isRoot()) {
             cleanTask.delete(project.file(project.getName() + ".xcworkspace"));
         }
-        getCleanTask().dependsOn(cleanTask);
+        getCleanTask().configure(Actions.composite(withDescription("Cleans XCode project files (xcodeproj)"), dependsOn(cleanTask)));
     }
 
     private GenerateXcodeProjectFileTask createProjectTask(final ProjectInternal project) {
