@@ -191,7 +191,7 @@ public class OutputScrapingExecutionResult implements ExecutionResult {
 
     @Override
     public boolean hasErrorOutput(String expectedOutput) {
-        return getError().contains(expectedOutput);
+        return getError().contains(expectedOutput) || getRawError().contains(expectedOutput);
     }
 
     @Override
@@ -213,6 +213,10 @@ public class OutputScrapingExecutionResult implements ExecutionResult {
         return error.withNormalizedEol();
     }
 
+    public String getRawError() {
+        return errorContent.getRawContent().withNormalizedEol();
+    }
+
     public List<String> getExecutedTasks() {
         return ImmutableList.copyOf(findExecutedTasksInOrderStarted());
     }
