@@ -15,9 +15,13 @@
  */
 package org.gradle.test.fixtures.server.http;
 
+import com.google.common.io.Files;
 import com.sun.net.httpserver.BasicAuthenticator;
 import com.sun.net.httpserver.HttpContext;
+import com.sun.net.httpserver.HttpExchange;
 import com.sun.net.httpserver.HttpServer;
+import org.gradle.api.Action;
+import org.gradle.internal.ErroringAction;
 import org.junit.rules.ExternalResource;
 
 import java.io.File;
@@ -51,7 +55,7 @@ public class BlockingHttpServer extends ExternalResource {
     private boolean running;
 
     public BlockingHttpServer() throws IOException {
-        this(30000);
+        this(120000);
     }
 
     public BlockingHttpServer(int timeoutMs) throws IOException {
@@ -91,7 +95,7 @@ public class BlockingHttpServer extends ExternalResource {
      */
     public String callFromBuild(String resource) {
         URI uri = uri(resource);
-        return "System.out.println(\"calling " + uri + "\"); try { new java.net.URL(\"" + uri + "\").openConnection().getContentLength(); } catch(Exception e) { throw new RuntimeException(e); }; System.out.println(\"response received\");";
+        return "System.out.println(\"calling " + uri + "\"); try { new java.net.URL(\"" + uri + "\").openConnection().getContentLength(); } catch(Exception e) { throw new RuntimeException(e); }; System.out.println(\"[G] response received\");";
     }
 
     /**
@@ -99,7 +103,7 @@ public class BlockingHttpServer extends ExternalResource {
      */
     public String callFromBuildUsingExpression(String expression) {
         String uriExpression = "\"" + getUri() + "/\" + " + expression;
-        return "System.out.println(\"calling \" + " + uriExpression + "); try { new java.net.URL(" + uriExpression + ").openConnection().getContentLength(); } catch(Exception e) { throw new RuntimeException(e); }; System.out.println(\"response received\");";
+        return "System.out.println(\"calling \" + " + uriExpression + "); try { new java.net.URL(" + uriExpression + ").openConnection().getContentLength(); } catch(Exception e) { throw new RuntimeException(e); }; System.out.println(\"[G] response received\");";
     }
 
     public void withBasicAuthentication(final String username, final String password) {
@@ -117,7 +121,7 @@ public class BlockingHttpServer extends ExternalResource {
     public void expectConcurrent(String... expectedRequests) {
         List<ResourceExpectation> expectations = new ArrayList<ResourceExpectation>();
         for (String call : expectedRequests) {
-            expectations.add(new SendFixedContent(call));
+            expectations.add(new ExpectGetAndSendFixedContent(call));
         }
         addNonBlockingHandler(expectations);
     }
@@ -128,7 +132,7 @@ public class BlockingHttpServer extends ExternalResource {
     public void expectConcurrent(Collection<String> expectedRequests) {
         List<ResourceExpectation> expectations = new ArrayList<ResourceExpectation>();
         for (String call : expectedRequests) {
-            expectations.add(new SendFixedContent(call));
+            expectations.add(new ExpectGetAndSendFixedContent(call));
         }
         addNonBlockingHandler(expectations);
     }
@@ -156,36 +160,68 @@ public class BlockingHttpServer extends ExternalResource {
     /**
      * Expect a GET request to the given path, and return the contents of the given file.
      */
-    public ExpectedRequest file(String path, File file) {
-        return new SendFileContent(path, file);
+    public ExpectedRequest file(String path, final File file) {
+        return new ExpectMethodAndRunAction("GET", path, new ErroringAction<HttpExchange>() {
+            @Override
+            protected void doExecute(HttpExchange httpExchange) throws Exception {
+                httpExchange.sendResponseHeaders(200, file.length());
+                Files.copy(file, httpExchange.getResponseBody());
+            }
+        });
     }
 
     /**
      * Expect a GET request to the given path, and return some arbitrary content.
      */
     public ExpectedRequest resource(String path) {
-        return new SendFixedContent(path);
+        return new ExpectGetAndSendFixedContent(path);
     }
 
     /**
      * Expect a GET request to the given path, and return a 404 response.
      */
     public ExpectedRequest missing(String path) {
-        return new ExpectGetMissing(path);
+        return new ExpectMethodAndRunAction("GET", path, new ErroringAction<HttpExchange>() {
+            @Override
+            protected void doExecute(HttpExchange httpExchange) throws Exception {
+                httpExchange.sendResponseHeaders(404, 0);
+            }
+        });
+    }
+
+    /**
+     * Expect a HEAD request to the given path.
+     */
+    public ExpectedRequest head(String path) {
+        return new ExpectHead(path);
     }
 
     /**
      * Expect a GET request to the given path, and return the given content (UTF-8 encoded)
      */
     public ExpectedRequest resource(String path, String content) {
-        return new SendFixedContent(path, content);
+        return new ExpectGetAndSendFixedContent(path, content);
+    }
+
+    /**
+     * Expect a GET request to the given path and run the given action to create the response.
+     */
+    public ExpectedRequest get(String path, Action<? super HttpExchange> action) {
+        return new ExpectMethodAndRunAction("GET", path, action);
     }
 
     /**
      * Expect a PUT request to the given path, discard the request body
      */
     public ExpectedRequest put(String path) {
-        return new ExpectPut(path);
+        return new ExpectMethodAndRunAction("PUT", path, new SendEmptyResponse());
+    }
+
+    /**
+     * Expect a POST request to the given path and run the given action to create the response.
+     */
+    public ExpectedRequest post(String path, Action<? super HttpExchange> action) {
+        return new ExpectMethodAndRunAction("POST", path, action);
     }
 
     /**
@@ -212,11 +248,23 @@ public class BlockingHttpServer extends ExternalResource {
     public BlockingHandler expectConcurrentAndBlock(int concurrent, String... expectedCalls) {
         List<ResourceExpectation> expectations = new ArrayList<ResourceExpectation>();
         for (String call : expectedCalls) {
-            expectations.add(new SendFixedContent(call));
+            expectations.add(new ExpectGetAndSendFixedContent(call));
         }
         return addBlockingHandler(concurrent, expectations);
     }
 
+    /**
+     * Expects exactly the given number of calls to be made concurrently from any combination of the optionally expected calls. Blocks each call until they are explicitly released.
+     * Since the expectations are optional, they are still considered "complete" even if not all expected calls have been received.
+     */
+    public BlockingHandler expectOptionalAndBlock(int concurrent, String... optionalExpectedCalls) {
+        List<ResourceExpectation> expectations = new ArrayList<ResourceExpectation>();
+        for (String call : optionalExpectedCalls) {
+            expectations.add(new ExpectGetAndSendFixedContent(call));
+        }
+        return addBlockingOptionalHandler(concurrent, expectations);
+    }
+
     /**
      * Expects the given requests to be made concurrently. Blocks each request until they have all been received and released using one of the methods on {@link BlockingHandler}.
      */
@@ -245,18 +293,27 @@ public class BlockingHttpServer extends ExternalResource {
         });
     }
 
+    private BlockingHandler addBlockingOptionalHandler(final int concurrent, final Collection<? extends ResourceExpectation> expectations) {
+        return handler.addHandler(new ChainingHttpHandler.HandlerFactory<CyclicBarrierAnyOfRequestHandler>() {
+            @Override
+            public CyclicBarrierAnyOfRequestHandler create(WaitPrecondition previous) {
+                return new CyclicBarrierAnyOfOptionalRequestHandler(lock, serverId, timeoutMs, concurrent, previous, expectations);
+            }
+        });
+    }
+
     /**
      * Expects the given request to be made. Releases the request as soon as it is received.
      */
     public void expect(String expectedCall) {
-        addNonBlockingHandler(Collections.singleton(new SendFixedContent(expectedCall)));
+        addNonBlockingHandler(Collections.singleton(new ExpectGetAndSendFixedContent(expectedCall)));
     }
 
     /**
      * Expects the given request to be made. Blocks until the request is explicitly released using one of the methods on {@link BlockingHandler}.
      */
     public BlockingHandler expectAndBlock(String expectedCall) {
-        return addBlockingHandler(1, Collections.singleton(new SendFixedContent(expectedCall)));
+        return addBlockingHandler(1, Collections.singleton(new ExpectGetAndSendFixedContent(expectedCall)));
     }
 
     /**
@@ -302,7 +359,7 @@ public class BlockingHttpServer extends ExternalResource {
         stop();
     }
 
-    private int getPort() {
+    public int getPort() {
         if (!running) {
             throw new IllegalStateException("Cannot get HTTP port as server is not running.");
         }
@@ -355,6 +412,13 @@ public class BlockingHttpServer extends ExternalResource {
         void waitForAllPendingCalls();
     }
 
+    private static class SendEmptyResponse extends ErroringAction<HttpExchange> {
+        @Override
+        protected void doExecute(HttpExchange httpExchange) throws Exception {
+            httpExchange.sendResponseHeaders(200, 0);
+        }
+    }
+
     private class MustBeRunning implements WaitPrecondition {
         @Override
         public void assertCanWait() throws IllegalStateException {
