@@ -16,22 +16,19 @@
 
 package org.gradle.api.internal.tasks.compile.processing;
 
+import java.util.Locale;
+
 /**
  * The different kinds of annotation processors that the incremental compiler knows how to handle.
  * See the user guide chapter on incremental annotation processing for more information.
  */
 public enum IncrementalAnnotationProcessorType {
-    ISOLATING(true),
-    AGGREGATING(true),
-    UNKNOWN(false);
+    ISOLATING,
+    AGGREGATING,
+    DYNAMIC,
+    UNKNOWN;
 
-    private final boolean incremental;
-
-    IncrementalAnnotationProcessorType(boolean incremental) {
-        this.incremental = incremental;
-    }
-
-    public boolean isIncremental() {
-        return incremental;
+    public String getProcessorOption() {
+        return "org.gradle.annotation.processing." + name().toLowerCase(Locale.ROOT);
     }
 }
