@@ -17,10 +17,7 @@
 package org.gradle.language.nativeplatform.internal.incremental;
 
 import com.google.common.collect.ImmutableSet;
-import com.google.common.collect.Sets;
-import org.gradle.api.internal.changedetection.state.FileSnapshot;
 import org.gradle.api.internal.changedetection.state.FileSystemSnapshotter;
-import org.gradle.internal.file.FileType;
 import org.gradle.internal.hash.HashCode;
 import org.gradle.language.nativeplatform.internal.Include;
 import org.gradle.language.nativeplatform.internal.IncludeDirectives;
@@ -31,9 +28,11 @@ import org.slf4j.LoggerFactory;
 import javax.annotation.Nullable;
 import java.io.File;
 import java.util.ArrayList;
+import java.util.Collection;
 import java.util.Collections;
 import java.util.HashMap;
 import java.util.HashSet;
+import java.util.LinkedHashSet;
 import java.util.List;
 import java.util.Map;
 import java.util.Set;
@@ -55,7 +54,7 @@ public class IncrementalCompileFilesFactory {
         this.ignoreUnresolvedHeadersInDependencies = Boolean.getBoolean(IGNORE_UNRESOLVED_HEADERS_IN_DEPENDENCIES_PROPERTY_NAME);
     }
 
-    public IncementalCompileSourceProcessor filesFor(CompilationState previousCompileState) {
+    public IncementalCompileSourceProcessor files(CompilationState previousCompileState) {
         return new DefaultIncementalCompileSourceProcessor(previousCompileState);
     }
 
@@ -63,10 +62,8 @@ public class IncrementalCompileFilesFactory {
         private final CompilationState previous;
         private final BuildableCompilationState current = new BuildableCompilationState();
         private final List<File> toRecompile = new ArrayList<File>();
-        private final Set<File> existingHeaders = Sets.newHashSet();
-        private final Map<File, IncludeDirectives> includeDirectivesMap = new HashMap<File, IncludeDirectives>();
+        private final Set<File> existingHeaders = new HashSet<File>();
         private final Map<File, FileDetails> visitedFiles = new HashMap<File, FileDetails>();
-        int traversalCount;
         private boolean hasUnresolvedHeaders;
 
         DefaultIncementalCompileSourceProcessor(CompilationState previousCompileState) {
@@ -75,7 +72,7 @@ public class IncrementalCompileFilesFactory {
 
         @Override
         public IncrementalCompilation getResult() {
-            return new DefaultIncrementalCompilation(current.snapshot(), toRecompile, getRemovedSources(), existingHeaders, hasUnresolvedHeaders, includeDirectivesMap);
+            return new DefaultIncrementalCompilation(current.snapshot(), toRecompile, getRemovedSources(), existingHeaders, hasUnresolvedHeaders);
         }
 
         @Override
@@ -89,28 +86,80 @@ public class IncrementalCompileFilesFactory {
          * @return true if this source file requires recompilation, false otherwise.
          */
         private boolean visitSourceFile(File sourceFile) {
-            FileSnapshot fileSnapshot = fileSystemSnapshotter.snapshotSelf(sourceFile);
-            if (fileSnapshot.getType() != FileType.RegularFile) {
+            HashCode fileContent = fileSystemSnapshotter.getRegularFileContentHash(sourceFile);
+            if (fileContent == null) {
                 // Skip things that aren't files
                 return false;
             }
 
             SourceFileState previousState = previous.getState(sourceFile);
+
+            if (previousState != null) {
+                // Already seen this source file before. See if we can reuse the analysis from last time
+                if (graphHasNotChanged(sourceFile, fileContent, previousState, existingHeaders)) {
+                    // Include file graph for this source file has not changed, skip this file
+                    current.setState(sourceFile, previousState);
+                    if (previousState.isHasUnresolved() && !ignoreUnresolvedHeadersInDependencies) {
+                        hasUnresolvedHeaders = true;
+                        return true;
+                    }
+                    return false;
+                }
+                // Else, something has changed in the include file graph for this source file, so analyse again
+            }
+
+            // Source file has not been compiled before, or its include file graph has changed in some way
+            // Calculate the include file graph for the source file and mark for recompilation
+
             CollectingMacroLookup visibleMacros = new CollectingMacroLookup();
-            FileVisitResult result = visitFile(sourceFile, fileSnapshot, visibleMacros, new HashSet<HashCode>(), true);
-            ArrayList<IncludeFileState> includedFiles = new ArrayList<IncludeFileState>();
-            result.collectFilesInto(++traversalCount, includedFiles);
-            SourceFileState newState = new SourceFileState(fileSnapshot.getContent().getContentMd5(), ImmutableSet.copyOf(includedFiles));
+            FileVisitResult result = visitFile(sourceFile, fileContent, visibleMacros, new HashSet<HashCode>(), existingHeaders);
+            Set<IncludeFileEdge> includedFiles = new LinkedHashSet<IncludeFileEdge>();
+            result.collectFilesInto(includedFiles, new HashSet<File>());
+            SourceFileState newState = new SourceFileState(fileContent, result.result == IncludeFileResolutionResult.UnresolvedMacroIncludes, ImmutableSet.copyOf(includedFiles));
             current.setState(sourceFile, newState);
-            includeDirectivesMap.put(sourceFile, result.includeDirectives);
-            // Recompile this source file if:
-            // - we don't know how/whether it has been compiled before
-            // - the source or referenced include files contains include/import directives that cannot be resolved to an include file
-            // - the source file or sequence of included files have changed in some way (the order/set/cardinality of the files has changed or the content of any file has changed)
-            return previousState == null || result.result == IncludeFileResolutionResult.UnresolvedMacroIncludes || newState.hasChanged(previousState);
+            if (newState.isHasUnresolved()) {
+                hasUnresolvedHeaders = true;
+            }
+            return true;
         }
 
-        private FileVisitResult visitFile(File file, FileSnapshot fileSnapshot, CollectingMacroLookup visibleMacros, Set<HashCode> visited, boolean isSourceFile) {
+        private boolean graphHasNotChanged(File sourceFile, HashCode fileHash, SourceFileState previousState, Set<File> existingHeaders) {
+            if (!fileHash.equals(previousState.getHash())) {
+                // Source file has changed
+                return false;
+            }
+            if (previousState.getEdges().isEmpty()) {
+                // Source file has not changed and no include files
+                return true;
+            }
+
+            // Check each unique edge in the include file graph
+            Map<HashCode, File> includes = new HashMap<HashCode, File>(previousState.getEdges().size());
+            Set<File> headers = new HashSet<File>();
+            includes.put(fileHash, sourceFile);
+            for (IncludeFileEdge includeFileEdge : previousState.getEdges()) {
+                File includedFrom = includeFileEdge.getIncludedBy() != null ? includes.get(includeFileEdge.getIncludedBy()) : null;
+                SourceIncludesResolver.IncludeFile includeFile = sourceIncludesResolver.resolveInclude(includedFrom, includeFileEdge.getIncludePath());
+                if (includeFile == null) {
+                    // Include file not found (but previously was found)
+                    return false;
+                }
+                HashCode hash = includeFile.getContentHash();
+                if (!hash.equals(includeFileEdge.getResolvedTo())) {
+                    // Include file changed
+                    return false;
+                }
+                if (!existingHeaders.contains(includeFile.getFile())) {
+                    // Collect for later, do not add until the graph is known to have not changed
+                    headers.add(includeFile.getFile());
+                }
+                includes.put(hash, includeFile.getFile());
+            }
+            existingHeaders.addAll(headers);
+            return true;
+        }
+
+        private FileVisitResult visitFile(File file, HashCode newHash, CollectingMacroLookup visibleMacros, Set<HashCode> visited, Set<File> existingHeaders) {
             FileDetails fileDetails = visitedFiles.get(file);
             if (fileDetails != null && fileDetails.results != null) {
                 // A file that we can safely reuse the result for
@@ -118,7 +167,6 @@ public class IncrementalCompileFilesFactory {
                 return fileDetails.results;
             }
 
-            HashCode newHash = fileSnapshot.getContent().getContentMd5();
             if (!visited.add(newHash)) {
                 // A cycle, treat as resolved here
                 return new FileVisitResult(file);
@@ -126,39 +174,41 @@ public class IncrementalCompileFilesFactory {
 
             if (fileDetails == null) {
                 IncludeDirectives includeDirectives = sourceIncludesParser.parseIncludes(file);
-                fileDetails = new FileDetails(new IncludeFileState(newHash, file), includeDirectives);
+                fileDetails = new FileDetails(includeDirectives);
                 visitedFiles.put(file, fileDetails);
             }
 
             CollectingMacroLookup includedFileDirectives = new CollectingMacroLookup();
             visibleMacros.append(file, fileDetails.directives);
 
-            List<FileVisitResult> included = new ArrayList<FileVisitResult>(fileDetails.directives.getAll().size());
+            List<Include> allIncludes = fileDetails.directives.getAll();
+            List<FileVisitResult> included = allIncludes.isEmpty() ? Collections.<FileVisitResult>emptyList() : new ArrayList<FileVisitResult>(allIncludes.size());
+            List<IncludeFileEdge> edges = allIncludes.isEmpty() ? Collections.<IncludeFileEdge>emptyList() : new ArrayList<IncludeFileEdge>(allIncludes.size());
             IncludeFileResolutionResult result = IncludeFileResolutionResult.NoMacroIncludes;
-            for (Include include : fileDetails.directives.getAll()) {
+            for (Include include : allIncludes) {
                 if (include.getType() == IncludeType.MACRO && result == IncludeFileResolutionResult.NoMacroIncludes) {
                     result = IncludeFileResolutionResult.HasMacroIncludes;
                 }
                 SourceIncludesResolver.IncludeResolutionResult resolutionResult = sourceIncludesResolver.resolveInclude(file, include, visibleMacros);
                 if (!resolutionResult.isComplete()) {
                     LOGGER.info("Cannot locate header file for '{}' in source file '{}'. Assuming changed.", include.getAsSourceText(), file.getName());
-                    if (isSourceFile || !ignoreUnresolvedHeadersInDependencies) {
-                        hasUnresolvedHeaders = true;
+                    if (!ignoreUnresolvedHeadersInDependencies) {
                         result = IncludeFileResolutionResult.UnresolvedMacroIncludes;
                     }
                 }
                 for (SourceIncludesResolver.IncludeFile includeFile : resolutionResult.getFiles()) {
                     existingHeaders.add(includeFile.getFile());
-                    FileVisitResult includeVisitResult = visitFile(includeFile.getFile(), includeFile.getSnapshot(), visibleMacros, visited, false);
+                    FileVisitResult includeVisitResult = visitFile(includeFile.getFile(), includeFile.getContentHash(), visibleMacros, visited, existingHeaders);
                     if (includeVisitResult.result.ordinal() > result.ordinal()) {
                         result = includeVisitResult.result;
                     }
                     includeVisitResult.collectDependencies(includedFileDirectives);
                     included.add(includeVisitResult);
+                    edges.add(new IncludeFileEdge(includeFile.getPath(), includeFile.isQuotedInclude() ? newHash : null, includeFile.getContentHash()));
                 }
             }
 
-            FileVisitResult visitResult = new FileVisitResult(file, result, fileDetails.state, fileDetails.directives, included, includedFileDirectives);
+            FileVisitResult visitResult = new FileVisitResult(file, result, fileDetails.directives, included, edges, includedFileDirectives);
             if (result == IncludeFileResolutionResult.NoMacroIncludes) {
                 // No macro includes were seen in the include graph of this file, so the result can be reused if this file is seen again
                 fileDetails.results = visitResult;
@@ -187,14 +237,12 @@ public class IncrementalCompileFilesFactory {
      * Details of a file that are independent of where the file appears in the file include graph.
      */
     private static class FileDetails {
-        final IncludeFileState state;
         final IncludeDirectives directives;
         // Non-null when the result of visiting this file can be reused
         @Nullable
         FileVisitResult results;
 
-        FileDetails(IncludeFileState state, IncludeDirectives directives) {
-            this.state = state;
+        FileDetails(IncludeDirectives directives) {
             this.directives = directives;
         }
     }
@@ -205,55 +253,47 @@ public class IncrementalCompileFilesFactory {
     private static class FileVisitResult implements CollectingMacroLookup.MacroSource {
         private final File file;
         private final IncludeFileResolutionResult result;
-        private final IncludeFileState fileState;
         private final IncludeDirectives includeDirectives;
         private final List<FileVisitResult> included;
+        private final List<IncludeFileEdge> edges;
         private final CollectingMacroLookup includeFileDirectives;
-        int traversalCount;
 
-        FileVisitResult(File file, IncludeFileResolutionResult result, IncludeFileState fileState, IncludeDirectives includeDirectives, List<FileVisitResult> included, CollectingMacroLookup dependentIncludeDirectives) {
+        FileVisitResult(File file, IncludeFileResolutionResult result, IncludeDirectives includeDirectives, List<FileVisitResult> included, List<IncludeFileEdge> edges, CollectingMacroLookup dependentIncludeDirectives) {
             this.file = file;
             this.result = result;
-            this.fileState = fileState;
             this.includeDirectives = includeDirectives;
             this.included = included;
+            this.edges = edges;
             this.includeFileDirectives = dependentIncludeDirectives;
         }
 
         FileVisitResult(File file) {
             this.file = file;
             result = IncludeFileResolutionResult.NoMacroIncludes;
-            fileState = null;
             includeDirectives = null;
             included = Collections.emptyList();
+            edges = Collections.emptyList();
             includeFileDirectives = null;
         }
 
         void collectDependencies(CollectingMacroLookup directives) {
-            if (fileState != null) {
+            if (includeDirectives != null) {
                 directives.append(this);
             }
         }
 
-        void collectFilesInto(int traversal, List<IncludeFileState> files) {
-            if (traversalCount == traversal) {
-                // Already seen during this traversal, skip
-                return;
-            }
-
-            // Collect files
-            traversalCount = traversal;
-            if (fileState != null) {
-                files.add(fileState);
+        void collectFilesInto(Collection<IncludeFileEdge> files, Set<File> seen) {
+            if (includeDirectives != null && seen.add(file)) {
+                files.addAll(edges);
                 for (FileVisitResult include : included) {
-                    include.collectFilesInto(traversal, files);
+                    include.collectFilesInto(files, seen);
                 }
             }
         }
 
         @Override
         public void collectInto(CollectingMacroLookup lookup) {
-            if (fileState != null) {
+            if (includeDirectives != null) {
                 lookup.append(file, includeDirectives);
                 includeFileDirectives.appendTo(lookup);
             }
