@@ -16,10 +16,12 @@
 
 package org.gradle.launcher.exec;
 
+import org.gradle.api.internal.StartParameterInternal;
 import org.gradle.composite.internal.IncludedBuildControllers;
 import org.gradle.internal.invocation.BuildAction;
 import org.gradle.internal.invocation.BuildActionRunner;
 import org.gradle.internal.invocation.BuildController;
+import org.gradle.internal.jvm.UnsupportedJavaRuntimeException;
 import org.gradle.internal.operations.BuildOperationContext;
 import org.gradle.internal.operations.BuildOperationDescriptor;
 import org.gradle.internal.operations.BuildOperationExecutor;
@@ -43,6 +45,7 @@ public class RunAsBuildOperationBuildActionRunner implements BuildActionRunner {
         buildOperationExecutor.run(new RunnableBuildOperation() {
             @Override
             public void run(BuildOperationContext context) {
+                checkDeprecations((StartParameterInternal)buildController.getGradle().getStartParameter());
                 buildController.getGradle().getServices().get(IncludedBuildControllers.class).rootBuildOperationStarted();
                 delegate.run(action, buildController);
                 context.setResult(RESULT);
@@ -54,4 +57,11 @@ public class RunAsBuildOperationBuildActionRunner implements BuildActionRunner {
             }
         });
     }
+
+    private void checkDeprecations(StartParameterInternal startParameter) {
+        UnsupportedJavaRuntimeException.javaDeprecationWarning();
+
+        // This must be done here because DeprecationLogger needs to be initialized properly
+        startParameter.checkDeprecation();
+    }
 }
