@@ -16,6 +16,15 @@
 
 package org.gradle.internal.logging.events;
 
+import org.gradle.internal.scan.UsedByScanPlugin;
+
+@UsedByScanPlugin
 public interface OutputEventListener {
     void onOutput(OutputEvent event);
+
+    OutputEventListener NO_OP = new OutputEventListener() {
+        @Override
+        public void onOutput(OutputEvent event) {
+        }
+    };
 }
