@@ -17,12 +17,12 @@
 package org.gradle.api.publish.maven.tasks;
 
 import org.gradle.api.DefaultTask;
-import org.gradle.api.Incubating;
 import org.gradle.api.InvalidUserDataException;
 import org.gradle.api.file.FileCollection;
 import org.gradle.api.internal.artifacts.mvnsettings.LocalMavenRepositoryLocator;
 import org.gradle.api.publish.maven.MavenPublication;
 import org.gradle.api.publish.maven.internal.publication.MavenPublicationInternal;
+import org.gradle.api.tasks.Internal;
 import org.gradle.internal.Factory;
 import org.gradle.internal.logging.LoggingManagerInternal;
 
@@ -34,7 +34,6 @@ import java.util.concurrent.Callable;
  *
  * @since 2.4
  */
-@Incubating
 public abstract class AbstractPublishToMaven extends DefaultTask {
 
     private MavenPublicationInternal publication;
@@ -44,9 +43,9 @@ public abstract class AbstractPublishToMaven extends DefaultTask {
         getInputs().files(new Callable<FileCollection>() {
             public FileCollection call() throws Exception {
                 MavenPublicationInternal publicationInternal = getPublicationInternal();
-                return publicationInternal == null ? null : publicationInternal.getPublishableFiles();
+                return publicationInternal == null ? null : publicationInternal.getPublishableArtifacts().getFiles();
             }
-        });
+        }).withPropertyName("publication.publishableFiles");
 
         // Should repositories be able to participate in incremental?
         // At the least, they may be able to express themselves as output files
@@ -61,6 +60,7 @@ public abstract class AbstractPublishToMaven extends DefaultTask {
      *
      * @return The publication to be published
      */
+    @Internal
     public MavenPublication getPublication() {
         return publication;
     }
@@ -74,6 +74,7 @@ public abstract class AbstractPublishToMaven extends DefaultTask {
         this.publication = toPublicationInternal(publication);
     }
 
+    @Internal
     protected MavenPublicationInternal getPublicationInternal() {
         return toPublicationInternal(getPublication());
     }
