@@ -36,7 +36,6 @@ import org.gradle.internal.remote.internal.hub.protocol.InterHubMessage;
 import org.gradle.internal.serialize.SerializerRegistry;
 import org.gradle.internal.serialize.StatefulSerializer;
 import org.gradle.internal.serialize.kryo.TypeSafeSerializer;
-import org.gradle.util.CollectionUtils;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
@@ -53,13 +52,17 @@ public class MessageHubBackedObjectConnection implements ObjectConnection {
     //    private ClassLoader methodParamClassLoader;
     private List<SerializerRegistry> paramSerializers = new ArrayList<SerializerRegistry>();
     private Set<ClassLoader> methodParamClassLoaders = new HashSet<ClassLoader>();
+    private volatile boolean aborted;
 
     public MessageHubBackedObjectConnection(ExecutorFactory executorFactory, ConnectCompletion completion) {
-        this.hub = new MessageHub(completion.toString(), executorFactory, new Action<Throwable>() {
+        Action<Throwable> errorHandler = new Action<Throwable>() {
             public void execute(Throwable throwable) {
-                LOGGER.error("Unexpected exception thrown.", throwable);
+                if (!aborted) {
+                    LOGGER.error("Unexpected exception thrown.", throwable);
+                }
             }
-        });
+        };
+        this.hub = new MessageHub(completion.toString(), executorFactory, errorHandler);
         this.completion = completion;
     }
 
@@ -76,10 +79,7 @@ public class MessageHubBackedObjectConnection implements ObjectConnection {
         if (type.getClassLoader() != getClass().getClassLoader()) {
             methodParamClassLoaders.add(type.getClassLoader());
         }
-        Dispatch<MethodInvocation> handler = new ReflectionDispatch(instance);
-        if (instance instanceof StreamCompletion) {
-            handler = new BoundedDispatchWrapper((StreamCompletion) instance, handler);
-        }
+        Dispatch<MethodInvocation> handler = new DispatchWrapper<T>(instance);
         hub.addHandler(type.getName(), handler);
     }
 
@@ -101,7 +101,7 @@ public class MessageHubBackedObjectConnection implements ObjectConnection {
         if (methodParamClassLoaders.size() == 0) {
             methodParamClassLoader = getClass().getClassLoader();
         } else if (methodParamClassLoaders.size() == 1) {
-            methodParamClassLoader = CollectionUtils.single(methodParamClassLoaders);
+            methodParamClassLoader = methodParamClassLoaders.iterator().next();
         } else {
             methodParamClassLoader = new CachingClassLoader(new MultiParentClassLoader(methodParamClassLoaders));
         }
@@ -128,23 +128,38 @@ public class MessageHubBackedObjectConnection implements ObjectConnection {
         CompositeStoppable.stoppable(hub, connection).stop();
     }
 
-    private static class BoundedDispatchWrapper implements BoundedDispatch<MethodInvocation> {
-        private final StreamCompletion instance;
+    @Override
+    public void abort() {
+        aborted = true;
+        stop();
+    }
+
+    private static class DispatchWrapper<T> implements BoundedDispatch<MethodInvocation>, StreamFailureHandler {
+        private final T instance;
         private final Dispatch<MethodInvocation> handler;
 
-        BoundedDispatchWrapper(StreamCompletion instance, Dispatch<MethodInvocation> handler) {
+        DispatchWrapper(T instance) {
             this.instance = instance;
-            this.handler = handler;
+            this.handler = new ReflectionDispatch(instance);
         }
 
         @Override
         public void endStream() {
-            instance.endStream();
+            if (instance instanceof StreamCompletion) {
+                ((StreamCompletion)instance).endStream();
+            }
         }
 
         @Override
         public void dispatch(MethodInvocation message) {
             handler.dispatch(message);
         }
+
+        @Override
+        public void handleStreamFailure(Throwable t) {
+            if (instance instanceof StreamFailureHandler) {
+                ((StreamFailureHandler)instance).handleStreamFailure(t);
+            }
+        }
     }
 }
