@@ -15,9 +15,12 @@
  */
 package org.gradle.cache.internal;
 
+import com.google.common.annotations.VisibleForTesting;
+import org.gradle.api.Action;
 import org.gradle.cache.FileIntegrityViolationException;
 import org.gradle.cache.FileLock;
 import org.gradle.cache.FileLockManager;
+import org.gradle.cache.FileLockReleasedSignal;
 import org.gradle.cache.InsufficientLockModeException;
 import org.gradle.cache.LockOptions;
 import org.gradle.cache.LockTimeoutException;
@@ -46,7 +49,11 @@ import java.io.IOException;
 import java.util.Random;
 import java.util.Set;
 import java.util.concurrent.CopyOnWriteArraySet;
+import java.util.concurrent.locks.Condition;
+import java.util.concurrent.locks.Lock;
+import java.util.concurrent.locks.ReentrantLock;
 
+import static java.util.concurrent.TimeUnit.MILLISECONDS;
 import static org.gradle.internal.UncheckedException.throwAsUncheckedException;
 
 /**
@@ -87,7 +94,7 @@ public class DefaultFileLockManager implements FileLockManager {
         return lock(target, options, targetDisplayName, operationDisplayName, null);
     }
 
-    public FileLock lock(File target, LockOptions options, String targetDisplayName, String operationDisplayName, Runnable whenContended) {
+    public FileLock lock(File target, LockOptions options, String targetDisplayName, String operationDisplayName, Action<FileLockReleasedSignal> whenContended) {
         if (options.getMode() == LockMode.None) {
             throw new UnsupportedOperationException(String.format("No %s mode lock implementation available.", options));
         }
@@ -104,6 +111,14 @@ public class DefaultFileLockManager implements FileLockManager {
         }
     }
 
+    static File determineLockTargetFile(File target) {
+        if (target.isDirectory()) {
+            return new File(target, target.getName() + ".lock");
+        } else {
+            return new File(target.getParentFile(), target.getName() + ".lock");
+        }
+    }
+
     private class DefaultFileLock extends AbstractFileAccess implements FileLock {
         private final File lockFile;
         private final File target;
@@ -116,7 +131,7 @@ public class DefaultFileLockManager implements FileLockManager {
         private int port;
         private final long lockId;
 
-        public DefaultFileLock(File target, LockOptions options, String displayName, String operationDisplayName, int port, Runnable whenContended) throws Throwable {
+        public DefaultFileLock(File target, LockOptions options, String displayName, String operationDisplayName, int port, Action<FileLockReleasedSignal> whenContended) throws Throwable {
             this.port = port;
             this.lockId = generator.generateId();
             if (options.getMode() == LockMode.None) {
@@ -127,11 +142,7 @@ public class DefaultFileLockManager implements FileLockManager {
 
             this.displayName = displayName;
             this.operationDisplayName = operationDisplayName;
-            if (target.isDirectory()) {
-                lockFile = new File(target, target.getName() + ".lock");
-            } else {
-                lockFile = new File(target.getParentFile(), target.getName() + ".lock");
-            }
+            this.lockFile = determineLockTargetFile(target);
 
             GFileUtils.mkdirs(lockFile.getParentFile());
             try {
@@ -348,7 +359,7 @@ public class DefaultFileLockManager implements FileLockManager {
                                 lastLockHolderPort = lockInfo.port;
                                 lastPingTime = 0;
                             }
-                            if (fileLockContentionHandler.maybePingOwner(lockInfo.port, lockInfo.lockId, displayName, backoff.timer.getElapsedMillis() - lastPingTime)) {
+                            if (fileLockContentionHandler.maybePingOwner(lockInfo.port, lockInfo.lockId, displayName, backoff.timer.getElapsedMillis() - lastPingTime, backoff.signal)) {
                                 lastPingTime = backoff.timer.getElapsedMillis();
                                 LOGGER.debug("The file lock is held by a different Gradle process (pid: {}, lockId: {}). Pinged owner at port {}", lockInfo.pid, lockInfo.lockId, lockInfo.port);
                             }
@@ -378,10 +389,10 @@ public class DefaultFileLockManager implements FileLockManager {
     private static class ExponentialBackoff {
 
         private static final int CAP_FACTOR = 100;
-
-        private static final long SLOT_TIME = 25L;
+        private static final long SLOT_TIME = 25;
 
         private final Random random = new Random();
+        private final AwaitableFileLockReleasedSignal signal = new AwaitableFileLockReleasedSignal();
 
         private final int timeoutMs;
         private CountdownTimer timer;
@@ -402,7 +413,10 @@ public class DefaultFileLockManager implements FileLockManager {
                 if (timer.hasExpired()) {
                     break;
                 }
-                Thread.sleep(backoffPeriodFor(++iteration));
+                boolean signaled = signal.await(backoffPeriodFor(++iteration));
+                if (signaled) {
+                    iteration = 0;
+                }
             }
             return result;
         }
@@ -411,4 +425,45 @@ public class DefaultFileLockManager implements FileLockManager {
             return random.nextInt(Math.min(iteration, CAP_FACTOR)) * SLOT_TIME;
         }
     }
+
+    @VisibleForTesting
+    static class AwaitableFileLockReleasedSignal implements FileLockReleasedSignal {
+
+        private final Lock lock = new ReentrantLock();
+        private final Condition condition = lock.newCondition();
+        private int waiting;
+
+        public boolean await(long millis) throws InterruptedException {
+            lock.lock();
+            try {
+                waiting++;
+                return condition.await(millis, MILLISECONDS);
+            } finally {
+                waiting--;
+                lock.unlock();
+            }
+        }
+
+        @Override
+        public void trigger() {
+            lock.lock();
+            try {
+                if (waiting > 0) {
+                    condition.signalAll();
+                }
+            } finally {
+                lock.unlock();
+            }
+        }
+
+        @VisibleForTesting
+        boolean isWaiting() {
+            lock.lock();
+            try {
+                return waiting > 0;
+            } finally {
+                lock.unlock();
+            }
+        }
+    }
 }
