@@ -26,7 +26,6 @@ import org.gradle.nativeplatform.internal.BinaryToolSpec;
 import org.gradle.nativeplatform.internal.CompilerOutputFileNamingSchemeFactory;
 import org.gradle.nativeplatform.internal.LinkerSpec;
 import org.gradle.nativeplatform.internal.StaticLibraryArchiverSpec;
-import org.gradle.nativeplatform.platform.internal.NativePlatformInternal;
 import org.gradle.nativeplatform.platform.internal.OperatingSystemInternal;
 import org.gradle.nativeplatform.toolchain.internal.AbstractPlatformToolProvider;
 import org.gradle.nativeplatform.toolchain.internal.CommandLineToolContext;
@@ -50,7 +49,6 @@ import org.gradle.nativeplatform.toolchain.internal.compilespec.ObjectiveCPCHCom
 import org.gradle.nativeplatform.toolchain.internal.compilespec.ObjectiveCppCompileSpec;
 import org.gradle.nativeplatform.toolchain.internal.compilespec.ObjectiveCppPCHCompileSpec;
 import org.gradle.nativeplatform.toolchain.internal.gcc.metadata.GccMetadata;
-import org.gradle.nativeplatform.toolchain.internal.gcc.metadata.SystemLibraryDiscovery;
 import org.gradle.nativeplatform.toolchain.internal.metadata.CompilerMetaDataProvider;
 import org.gradle.nativeplatform.toolchain.internal.metadata.CompilerMetadata;
 import org.gradle.nativeplatform.toolchain.internal.tools.CommandLineToolSearchResult;
@@ -59,7 +57,6 @@ import org.gradle.nativeplatform.toolchain.internal.tools.ToolRegistry;
 import org.gradle.nativeplatform.toolchain.internal.tools.ToolSearchPath;
 import org.gradle.platform.base.internal.toolchain.ComponentNotFound;
 import org.gradle.platform.base.internal.toolchain.SearchResult;
-import org.gradle.platform.base.internal.toolchain.ToolSearchResult;
 import org.gradle.process.internal.ExecActionFactory;
 
 import java.util.Collections;
@@ -76,18 +73,15 @@ class GccPlatformToolProvider extends AbstractPlatformToolProvider {
     );
 
     private final ToolSearchPath toolSearchPath;
-    private final NativePlatformInternal targetPlatform;
     private final ToolRegistry toolRegistry;
     private final ExecActionFactory execActionFactory;
     private final CompilerOutputFileNamingSchemeFactory compilerOutputFileNamingSchemeFactory;
     private final boolean useCommandFile;
     private final WorkerLeaseService workerLeaseService;
     private final CompilerMetaDataProvider<GccMetadata> metadataProvider;
-    private final SystemLibraryDiscovery standardLibraryDiscovery;
 
-    GccPlatformToolProvider(NativePlatformInternal targetPlatform, BuildOperationExecutor buildOperationExecutor, OperatingSystemInternal targetOperatingSystem, ToolSearchPath toolSearchPath, ToolRegistry toolRegistry, ExecActionFactory execActionFactory, CompilerOutputFileNamingSchemeFactory compilerOutputFileNamingSchemeFactory, boolean useCommandFile, WorkerLeaseService workerLeaseService, CompilerMetaDataProvider<GccMetadata> metadataProvider, SystemLibraryDiscovery standardLibraryDiscovery) {
+    GccPlatformToolProvider(BuildOperationExecutor buildOperationExecutor, OperatingSystemInternal targetOperatingSystem, ToolSearchPath toolSearchPath, ToolRegistry toolRegistry, ExecActionFactory execActionFactory, CompilerOutputFileNamingSchemeFactory compilerOutputFileNamingSchemeFactory, boolean useCommandFile, WorkerLeaseService workerLeaseService, CompilerMetaDataProvider<GccMetadata> metadataProvider) {
         super(buildOperationExecutor, targetOperatingSystem);
-        this.targetPlatform = targetPlatform;
         this.toolRegistry = toolRegistry;
         this.toolSearchPath = toolSearchPath;
         this.compilerOutputFileNamingSchemeFactory = compilerOutputFileNamingSchemeFactory;
@@ -95,12 +89,11 @@ class GccPlatformToolProvider extends AbstractPlatformToolProvider {
         this.execActionFactory = execActionFactory;
         this.workerLeaseService = workerLeaseService;
         this.metadataProvider = metadataProvider;
-        this.standardLibraryDiscovery = standardLibraryDiscovery;
     }
 
     @Override
-    public ToolSearchResult isToolAvailable(ToolType toolType) {
-        return toolSearchPath.locate(toolType, toolRegistry.getTool(toolType).getExecutable());
+    public CommandLineToolSearchResult locateTool(ToolType compilerType) {
+        return toolSearchPath.locate(compilerType, toolRegistry.getTool(compilerType).getExecutable());
     }
 
     @Override
