@@ -19,6 +19,7 @@ package org.gradle.api.plugins;
 import org.gradle.api.Project;
 import org.gradle.api.internal.file.FileLookup;
 import org.gradle.api.internal.project.ProjectInternal;
+import org.gradle.util.DeprecationLogger;
 
 import java.io.File;
 
@@ -32,8 +33,21 @@ public class BasePluginConvention {
 
     private String libsDirName;
 
+    // cached resolved values
+    private File buildDir;
+    private File libsDir;
+    private File distsDir;
+
     private String archivesBaseName;
+
+    /**
+     * Creates a {@link BasePluginConvention} instance.
+     *
+     * @deprecated Creating instances of this class is deprecated. These should be created by the base plugin only.
+     */
+    @Deprecated
     public BasePluginConvention(Project project) {
+        DeprecationLogger.nagUserOfDeprecated("Creating instances of BasePluginConvention");
         this.project = (ProjectInternal) project;
         archivesBaseName = project.getName();
         distsDirName = "distributions";
@@ -46,7 +60,14 @@ public class BasePluginConvention {
      * @return The directory. Never returns null.
      */
     public File getDistsDir() {
-        return project.getServices().get(FileLookup.class).getFileResolver(project.getBuildDir()).resolve(distsDirName);
+        File curProjectBuildDir = project.getBuildDir();
+        if (distsDir != null && curProjectBuildDir.equals(buildDir)) {
+            return distsDir;
+        }
+        buildDir = curProjectBuildDir;
+        File dir = project.getServices().get(FileLookup.class).getFileResolver(curProjectBuildDir).resolve(distsDirName);
+        distsDir = dir;
+        return dir;
     }
 
     /**
@@ -55,7 +76,14 @@ public class BasePluginConvention {
      * @return The directory. Never returns null.
      */
     public File getLibsDir() {
-        return project.getServices().get(FileLookup.class).getFileResolver(project.getBuildDir()).resolve(libsDirName);
+        File curProjectBuildDir = project.getBuildDir();
+        if (libsDir != null && curProjectBuildDir.equals(buildDir)) {
+            return libsDir;
+        }
+        buildDir = curProjectBuildDir;
+        File dir = project.getServices().get(FileLookup.class).getFileResolver(curProjectBuildDir).resolve(libsDirName);
+        libsDir = dir;
+        return dir;
     }
 
     public ProjectInternal getProject() {
@@ -75,6 +103,7 @@ public class BasePluginConvention {
 
     public void setDistsDirName(String distsDirName) {
         this.distsDirName = distsDirName;
+        this.distsDir = null;
     }
 
     /**
@@ -86,6 +115,7 @@ public class BasePluginConvention {
 
     public void setLibsDirName(String libsDirName) {
         this.libsDirName = libsDirName;
+        this.libsDir = null;
     }
 
     /**
