@@ -90,7 +90,7 @@ public class PublishingPlugin implements Plugin<Project> {
             return DefaultPublishingExtension.class;
         } else {
             DeprecationLogger.nagUserWithDeprecatedBuildInvocationFeature(
-                "As part of making the publishing plugins stable, the 'deferred configurable' behavior of the 'publishing {}' block is now deprecated.",
+                "As part of making the publishing plugins stable, the 'deferred configurable' behavior of the 'publishing {}' block",
                 "In Gradle 5.0 the 'enableFeaturePreview('STABLE_PUBLISHING')' flag will be removed and the new behavior will become the default.",
                     "Please add 'enableFeaturePreview('STABLE_PUBLISHING')' to your settings file and do a test run by publishing to a local repository. " +
                     "If all artifacts are published as expected, there is nothing else to do. " +
