@@ -16,18 +16,28 @@
 package org.gradle.api.plugins.scala;
 
 import com.google.common.annotations.VisibleForTesting;
+import com.google.common.collect.ImmutableSet;
 import org.codehaus.groovy.runtime.InvokerHelper;
 import org.gradle.api.Action;
+import org.gradle.api.ActionConfiguration;
 import org.gradle.api.Plugin;
 import org.gradle.api.Project;
 import org.gradle.api.Task;
+import org.gradle.api.artifacts.ArtifactView;
 import org.gradle.api.artifacts.Configuration;
 import org.gradle.api.artifacts.DependencySet;
+import org.gradle.api.artifacts.component.ComponentIdentifier;
+import org.gradle.api.artifacts.component.ProjectComponentIdentifier;
+import org.gradle.api.attributes.AttributeDisambiguationRule;
+import org.gradle.api.attributes.AttributeMatchingStrategy;
+import org.gradle.api.attributes.MultipleCandidatesDetails;
+import org.gradle.api.attributes.Usage;
 import org.gradle.api.file.FileCollection;
 import org.gradle.api.file.FileTreeElement;
 import org.gradle.api.file.SourceDirectorySet;
 import org.gradle.api.internal.file.SourceDirectorySetFactory;
 import org.gradle.api.internal.tasks.DefaultScalaSourceSet;
+import org.gradle.api.model.ObjectFactory;
 import org.gradle.api.plugins.Convention;
 import org.gradle.api.plugins.JavaBasePlugin;
 import org.gradle.api.plugins.JavaPluginConvention;
@@ -56,24 +66,32 @@ public class ScalaBasePlugin implements Plugin<Project> {
     @VisibleForTesting
     public static final String ZINC_CONFIGURATION_NAME = "zinc";
     public static final String SCALA_RUNTIME_EXTENSION_NAME = "scalaRuntime";
+
     private final SourceDirectorySetFactory sourceDirectorySetFactory;
+    private final ObjectFactory objectFactory;
 
     @Inject
-    public ScalaBasePlugin(SourceDirectorySetFactory sourceDirectorySetFactory) {
+    public ScalaBasePlugin(SourceDirectorySetFactory sourceDirectorySetFactory, ObjectFactory objectFactory) {
         this.sourceDirectorySetFactory = sourceDirectorySetFactory;
+        this.objectFactory = objectFactory;
     }
 
-    public void apply(Project project) {
+    public void apply(final Project project) {
         project.getPluginManager().apply(JavaBasePlugin.class);
 
-        configureConfigurations(project);
-        ScalaRuntime scalaRuntime = configureScalaRuntimeExtension(project);
+        Usage incrementalAnalysisUsage = objectFactory.named(Usage.class, "incremental-analysis");
+        configureConfigurations(project, incrementalAnalysisUsage);
+
+        ScalaRuntime scalaRuntime = project.getExtensions().create(SCALA_RUNTIME_EXTENSION_NAME, ScalaRuntime.class, project);
+
         configureCompileDefaults(project, scalaRuntime);
-        configureSourceSetDefaults(project, sourceDirectorySetFactory);
+
+        configureSourceSetDefaults(project, incrementalAnalysisUsage, sourceDirectorySetFactory);
+
         configureScaladoc(project, scalaRuntime);
     }
 
-    private static void configureConfigurations(final Project project) {
+    private void configureConfigurations(final Project project, final Usage incrementalAnalysisUsage) {
         project.getConfigurations().create(ZINC_CONFIGURATION_NAME).setVisible(false).setDescription("The Zinc incremental compiler to be used for this Scala project.")
             .defaultDependencies(new Action<DependencySet>() {
                 @Override
@@ -81,13 +99,26 @@ public class ScalaBasePlugin implements Plugin<Project> {
                     dependencies.add(project.getDependencies().create("com.typesafe.zinc:zinc:" + DefaultScalaToolProvider.DEFAULT_ZINC_VERSION));
                 }
             });
+
+        final Configuration incrementalAnalysisElements = project.getConfigurations().create("incrementalScalaAnalysisElements");
+        incrementalAnalysisElements.setVisible(false);
+        incrementalAnalysisElements.setDescription("Incremental compilation analysis files");
+        incrementalAnalysisElements.setCanBeResolved(false);
+        incrementalAnalysisElements.setCanBeConsumed(true);
+        incrementalAnalysisElements.getAttributes().attribute(Usage.USAGE_ATTRIBUTE, incrementalAnalysisUsage);
+
+        AttributeMatchingStrategy<Usage> matchingStrategy = project.getDependencies().getAttributesSchema().attribute(Usage.USAGE_ATTRIBUTE);
+        matchingStrategy.getDisambiguationRules().add(UsageDisambiguationRules.class, new Action<ActionConfiguration>() {
+            @Override
+            public void execute(ActionConfiguration actionConfiguration) {
+                actionConfiguration.params(incrementalAnalysisUsage);
+                actionConfiguration.params(objectFactory.named(Usage.class, Usage.JAVA_API));
+                actionConfiguration.params(objectFactory.named(Usage.class, Usage.JAVA_RUNTIME_JARS));
+            }
+        });
     }
 
-    private static ScalaRuntime configureScalaRuntimeExtension(Project project) {
-        return project.getExtensions().create(SCALA_RUNTIME_EXTENSION_NAME, ScalaRuntime.class, project);
-    }
-
-    private static void configureSourceSetDefaults(final Project project, final SourceDirectorySetFactory sourceDirectorySetFactory) {
+    private static void configureSourceSetDefaults(final Project project, final Usage incrementalAnalysisUsage, final SourceDirectorySetFactory sourceDirectorySetFactory) {
         project.getConvention().getPlugin(JavaPluginConvention.class).getSourceSets().all(new Action<SourceSet>() {
             @Override
             public void execute(final SourceSet sourceSet) {
@@ -95,13 +126,9 @@ public class ScalaBasePlugin implements Plugin<Project> {
                 Convention sourceSetConvention = (Convention) InvokerHelper.getProperty(sourceSet, "convention");
                 DefaultScalaSourceSet scalaSourceSet = new DefaultScalaSourceSet(displayName, sourceDirectorySetFactory);
                 sourceSetConvention.getPlugins().put("scala", scalaSourceSet);
+
                 final SourceDirectorySet scalaDirectorySet = scalaSourceSet.getScala();
-                scalaDirectorySet.srcDir(new Callable<File>() {
-                    @Override
-                    public File call() throws Exception {
-                        return project.file("src/" + sourceSet.getName() + "/scala");
-                    }
-                });
+                scalaDirectorySet.srcDir(project.file("src/" + sourceSet.getName() + "/scala"));
                 sourceSet.getAllJava().source(scalaDirectorySet);
                 sourceSet.getAllSource().source(scalaDirectorySet);
                 sourceSet.getResources().getFilter().exclude(new Spec<FileTreeElement>() {
@@ -111,16 +138,26 @@ public class ScalaBasePlugin implements Plugin<Project> {
                     }
                 });
 
-                configureScalaCompile(project, sourceSet);
+                Configuration classpath = project.getConfigurations().getByName(sourceSet.getImplementationConfigurationName());
+                Configuration incrementalAnalysis = project.getConfigurations().create("incrementalScalaAnalysisFor" + sourceSet.getName());
+                incrementalAnalysis.setVisible(false);
+                incrementalAnalysis.setDescription("Incremental compilation analysis files for " + displayName);
+                incrementalAnalysis.setCanBeResolved(true);
+                incrementalAnalysis.setCanBeConsumed(false);
+                incrementalAnalysis.extendsFrom(classpath);
+                incrementalAnalysis.getAttributes().attribute(Usage.USAGE_ATTRIBUTE, incrementalAnalysisUsage);
+
+                configureScalaCompile(project, sourceSet, incrementalAnalysis, incrementalAnalysisUsage);
             }
 
         });
     }
 
-    private static void configureScalaCompile(final Project project, final SourceSet sourceSet) {
+    private static void configureScalaCompile(final Project project, final SourceSet sourceSet, final Configuration incrementalAnalysis, final Usage incrementalAnalysisUsage) {
         Convention scalaConvention = (Convention) InvokerHelper.getProperty(sourceSet, "convention");
         final ScalaSourceSet scalaSourceSet = scalaConvention.findPlugin(ScalaSourceSet.class);
         SourceSetUtil.configureOutputDirectoryForSourceSet(sourceSet, scalaSourceSet.getScala(), project);
+
         final TaskProvider<ScalaCompile> scalaCompile = project.getTasks().register(sourceSet.getCompileTaskName("scala"), ScalaCompile.class, new Action<ScalaCompile>() {
             @Override
             public void execute(ScalaCompile scalaCompile) {
@@ -129,20 +166,35 @@ public class ScalaBasePlugin implements Plugin<Project> {
                 scalaCompile.setDescription("Compiles the " + scalaSourceSet.getScala() + ".");
                 scalaCompile.setSource(scalaSourceSet.getScala());
 
+                scalaCompile.getAnalysisMappingFile().set(project.getLayout().getBuildDirectory().file("tmp/scala/compilerAnalysis/" + scalaCompile.getName() + ".mapping"));
 
-                // cannot use convention mapping because the resulting object won't be serializable
                 // cannot compute at task execution time because we need association with source set
-                // TODO: Replace this with Providers
                 IncrementalCompileOptions incrementalOptions = scalaCompile.getScalaCompileOptions().getIncrementalOptions();
-                if (incrementalOptions.getAnalysisFile() == null) {
-                    String analysisFilePath = project.getBuildDir().getPath() + "/tmp/scala/compilerAnalysis/" + scalaCompile.getName() + ".analysis";
-                    incrementalOptions.setAnalysisFile(new File(analysisFilePath));
-                }
-
-                if (incrementalOptions.getPublishedCode() == null) {
-                    Jar jarTask = (Jar) project.getTasks().findByName(sourceSet.getJarTaskName());
-                    incrementalOptions.setPublishedCode(jarTask == null ? null : jarTask.getArchivePath());
+                incrementalOptions.getAnalysisFile().set(
+                    project.getLayout().getBuildDirectory().file("tmp/scala/compilerAnalysis/" + scalaCompile.getName() + ".analysis")
+                );
+                // TODO: Replace this with Providers in Jar/AbstractArchiveTask
+                final Jar jarTask = (Jar) project.getTasks().findByName(sourceSet.getJarTaskName());
+                if (jarTask != null) {
+                    incrementalOptions.getPublishedCode().set(project.getLayout().file(project.provider(new Callable<File>() {
+                        @Override
+                        public File call() throws Exception {
+                            return jarTask.getArchivePath();
+                        }
+                    })));
                 }
+                scalaCompile.getAnalysisFiles().from(incrementalAnalysis.getIncoming().artifactView(new Action<ArtifactView.ViewConfiguration>() {
+                    @Override
+                    public void execute(ArtifactView.ViewConfiguration viewConfiguration) {
+                        viewConfiguration.lenient(true);
+                        viewConfiguration.componentFilter(new Spec<ComponentIdentifier>() {
+                            @Override
+                            public boolean isSatisfiedBy(ComponentIdentifier element) {
+                                return element instanceof ProjectComponentIdentifier;
+                            }
+                        });
+                    }
+                }).getFiles());
             }
         });
 
@@ -200,4 +252,24 @@ public class ScalaBasePlugin implements Plugin<Project> {
             }
         });
     }
+
+    static class UsageDisambiguationRules implements AttributeDisambiguationRule<Usage> {
+        private final ImmutableSet<Usage> expectedUsages;
+        private final Usage javaRuntimeJars;
+
+        @Inject
+        UsageDisambiguationRules(Usage incrementalAnalysis, Usage javaApi, Usage javaRuntimeJars) {
+            this.javaRuntimeJars = javaRuntimeJars;
+            this.expectedUsages = ImmutableSet.of(incrementalAnalysis, javaApi, javaRuntimeJars);
+        }
+
+        @Override
+        public void execute(MultipleCandidatesDetails<Usage> details) {
+            if (details.getConsumerValue() == null) {
+                if (details.getCandidateValues().equals(expectedUsages)) {
+                    details.closestMatch(javaRuntimeJars);
+                }
+            }
+        }
+    }
 }
