@@ -21,10 +21,10 @@ import org.gradle.api.Incubating;
 import org.gradle.api.Transformer;
 import org.gradle.api.file.FileCollection;
 import org.gradle.api.internal.file.UnionFileCollection;
-import org.gradle.api.internal.tasks.testing.junit.report.DefaultTestReport;
 import org.gradle.api.internal.tasks.testing.junit.result.AggregateTestResultsProvider;
 import org.gradle.api.internal.tasks.testing.junit.result.BinaryResultBackedTestResultsProvider;
 import org.gradle.api.internal.tasks.testing.junit.result.TestResultsProvider;
+import org.gradle.api.internal.tasks.testing.report.DefaultTestReport;
 import org.gradle.api.tasks.InputFiles;
 import org.gradle.api.tasks.OutputDirectory;
 import org.gradle.api.tasks.SkipWhenEmpty;
@@ -83,14 +83,14 @@ public class TestReport extends DefaultTask {
     private void addTo(Object result, UnionFileCollection dirs) {
         if (result instanceof Test) {
             Test test = (Test) result;
-            dirs.add(getProject().files(test.getBinResultsDir()).builtBy(test));
+            dirs.addToUnion(getProject().files(test.getBinResultsDir()).builtBy(test));
         } else if (result instanceof Iterable<?>) {
             Iterable<?> iterable = (Iterable<?>) result;
             for (Object nested : iterable) {
                 addTo(nested, dirs);
             }
         } else {
-            dirs.add(getProject().files(result));
+            dirs.addToUnion(getProject().files(result));
         }
     }
 
