@@ -20,10 +20,15 @@ import groovy.lang.MissingMethodException;
 import groovy.lang.MissingPropertyException;
 import groovy.xml.MarkupBuilder;
 import org.gradle.api.Incubating;
+import org.gradle.api.tasks.Input;
+import org.gradle.api.tasks.InputFiles;
+import org.gradle.api.tasks.Optional;
+import org.gradle.api.tasks.OutputDirectory;
 import org.gradle.api.tasks.testing.TestFrameworkOptions;
 import org.gradle.internal.ErroringAction;
 import org.gradle.internal.IoActions;
 
+import javax.annotation.Nullable;
 import java.io.BufferedWriter;
 import java.io.File;
 import java.io.StringWriter;
@@ -115,7 +120,6 @@ public class TestNGOptions extends TestFrameworkOptions {
                 if (!buildSuiteXml.delete()) {
                     throw new RuntimeException("failed to remove already existing build-suite.xml file");
                 }
-
             }
 
             IoActions.writeTextFile(buildSuiteXml, new ErroringAction<BufferedWriter>() {
@@ -123,14 +127,13 @@ public class TestNGOptions extends TestFrameworkOptions {
                 protected void doExecute(BufferedWriter writer) throws Exception {
                     writer.write("<!DOCTYPE suite SYSTEM \"http://testng.org/testng-1.0.dtd\">");
                     writer.newLine();
-                    writer.write(suiteXmlWriter.toString());
+                    writer.write(getSuiteXml());
                 }
             });
 
             suites.add(buildSuiteXml);
         }
 
-
         return suites;
     }
 
@@ -176,6 +179,7 @@ public class TestNGOptions extends TestFrameworkOptions {
      * @since 1.11
      */
     @Incubating
+    @OutputDirectory
     public File getOutputDirectory() {
         return outputDirectory;
     }
@@ -188,6 +192,7 @@ public class TestNGOptions extends TestFrameworkOptions {
     /**
      * The set of groups to run.
      */
+    @Input
     public Set<String> getIncludeGroups() {
         return includeGroups;
     }
@@ -199,6 +204,7 @@ public class TestNGOptions extends TestFrameworkOptions {
     /**
      * The set of groups to exclude.
      */
+    @Input
     public Set<String> getExcludeGroups() {
         return excludeGroups;
     }
@@ -249,6 +255,7 @@ public class TestNGOptions extends TestFrameworkOptions {
      *
      * If not present, parallel mode will not be selected
      */
+    @Nullable
     public String getParallel() {
         return parallel;
     }
@@ -330,6 +337,7 @@ public class TestNGOptions extends TestFrameworkOptions {
      *
      * Note: The suiteXmlFiles can be used in conjunction with the suiteXmlBuilder.
      */
+    @InputFiles
     public List<File> getSuiteXmlFiles() {
         return suiteXmlFiles;
     }
@@ -383,6 +391,17 @@ public class TestNGOptions extends TestFrameworkOptions {
         this.groupByInstances = groupByInstances;
     }
 
+    /**
+     * Returns the XML generated using {@link #suiteXmlBuilder()}, if any.
+     *
+     * <p>This property is read-only and exists merely for up-to-date checking.
+     */
+    @Input
+    @Optional
+    private String getSuiteXml() {
+        return suiteXmlWriter == null ? null : suiteXmlWriter.toString();
+    }
+
     public StringWriter getSuiteXmlWriter() {
         return suiteXmlWriter;
     }
