@@ -16,8 +16,6 @@
 
 package org.gradle.vcs.git.internal;
 
-import org.gradle.StartParameter;
-import org.gradle.api.internal.initialization.ClassLoaderScope;
 import org.gradle.internal.UncheckedException;
 import org.gradle.vcs.git.GitVersionControlSpec;
 import org.gradle.vcs.internal.spec.AbstractVersionControlSpec;
@@ -28,10 +26,6 @@ import java.net.URISyntaxException;
 public class DefaultGitVersionControlSpec extends AbstractVersionControlSpec implements GitVersionControlSpec {
     private URI url;
 
-    public DefaultGitVersionControlSpec(StartParameter rootBuildStartParameter, ClassLoaderScope classLoaderScope) {
-        super(rootBuildStartParameter, classLoaderScope);
-    }
-
     @Override
     public URI getUrl() {
         return url;
@@ -54,7 +48,7 @@ public class DefaultGitVersionControlSpec extends AbstractVersionControlSpec imp
 
     @Override
     public String getDisplayName() {
-        return "Git Repository at " + getUrl();
+        return "Git repository at " + getUrl();
     }
 
     @Override
